@@ -84,6 +84,13 @@ let rec list_is_prefix l1 l2 =
   | a::ll1, b::ll2 when a=b -> list_is_prefix ll1 ll2
   | _ -> false
 
+let rec list_is_prefix_skip_end_slash l1 l2 =
+  match (l1, l2) with
+  | [""], _
+  | [], _ -> true
+  | a::ll1, b::ll2 when a=b -> list_is_prefix ll1 ll2
+  | _ -> false
+
 
 (** various functions for URLs *)
 
@@ -102,11 +109,13 @@ let remove_dotdot =
 
 let remove_slash_at_beginning = function
   | [] -> []
+  | [""] -> [""]
   | ""::l -> l
   | l -> l
 
 let rec recursively_remove_slash_at_beginning = function
   | [] -> []
+  | [""] -> [""]
   | ""::l -> recursively_remove_slash_at_beginning l
   | l -> l
 
@@ -245,12 +254,23 @@ let rec split ?(multisep=false) char s =
   aux 0
 
 (* printing exceptions *)
-let rec string_of_exn = function
-  | Dynlink.Error err ->
-      "Dynlink.Error: " ^ (Dynlink.error_message err)
-  | Unix.Unix_error (ee, func, param) ->
-      (Unix.error_message ee)^" in function "^func^" ("^param^")"
-  | e -> Printexc.to_string e
+
+let (string_of_exn, register_exn_printer) =
+  let current = ref
+    (fun f_rec -> function
+       | Unix.Unix_error (ee, func, param) ->
+           Printf.sprintf "%s in function %s (%s)"
+             (Unix.error_message ee) func param
+       | e -> Printexc.to_string e)
+  in
+  let rec string_of_exn e = !current string_of_exn e in
+  (string_of_exn,
+   (fun p ->
+      let old = !current in
+      current :=
+        (fun f_rec s ->
+           try p f_rec s
+           with e -> old f_rec s)))
 
 
 (* Unix.inet_addr is abstract and nothing to convert it :-( *)
@@ -403,9 +423,11 @@ let fst3 (a, _, _) = a
 let snd3 (_, a, _) = a
 let thd3 (_, _, a) = a
 
+exception No_such_host
+
 let get_inet_addr host =
   let rec aux = function
-    | [] -> Lwt.fail Not_found
+    | [] -> Lwt.fail No_such_host
     | {Unix.ai_addr=Unix.ADDR_INET (inet_addr, _)}::_ -> Lwt.return inet_addr
     | _::l -> aux l
   in
@@ -431,7 +453,7 @@ let getnameinfo ia p =
 (************************************************************************)
 (* URL parsing *)
 
-(* Ces deux trucs sont dans Neturl version 1.1.2 mais en attendant qu'ils
+(*VVV Ces deux trucs sont dans Neturl version 1.1.2 mais en attendant qu'ils
    soient dans debian, je les mets ici *)
 let problem_re = Netstring_pcre.regexp "[ <>\"{}|\\\\^\\[\\]`]"
 
@@ -442,40 +464,97 @@ let fixup_url_string =
        Printf.sprintf "%%%02x"
         (Char.code s.[Netstring_pcre.match_beginning m]))
 
-let string_of_url_path x = fixup_url_string (String.concat "/" x)
+(*VVV This is in Netencoding but we have a problem with ~ 
+  (not encoded by browsers). Here is a patch that does not encode '~': *)
+module MyUrl = struct
+  let hex_digits =
+    [| '0'; '1'; '2'; '3'; '4'; '5'; '6'; '7';
+       '8'; '9'; 'A'; 'B'; 'C'; 'D'; 'E'; 'F' |];;
+
+  let to_hex2 k =
+    (* Converts k to a 2-digit hex string *)
+    let s = String.create 2 in
+    s.[0] <- hex_digits.( (k lsr 4) land 15 );
+    s.[1] <- hex_digits.( k land 15 );
+    s ;;
+
+  let of_hex1 c =
+    match c with
+        ('0'..'9') -> Char.code c - Char.code '0'
+      | ('A'..'F') -> Char.code c - Char.code 'A' + 10
+      | ('a'..'f') -> Char.code c - Char.code 'a' + 10
+      | _ ->
+        raise Not_found ;;
+
+  let url_encoding_re =
+    Netstring_pcre.regexp "[^A-Za-z0-9_.!*-~]";;
+
+  let encode ?(plus = true) s =
+    Netstring_pcre.global_substitute
+      url_encoding_re
+      (fun r _ ->
+         match Netstring_pcre.matched_string r s with
+             " " when plus -> "+"
+           | x ->
+               let k = Char.code(x.[0]) in
+               "%" ^ to_hex2 k
+      )
+      s ;;
+end
+
+
+let string_of_url_path ~encode l = 
+  if encode
+  then
+    fixup_url_string (String.concat "/"
+                        (List.map (*Netencoding.Url.encode*) MyUrl.encode l))
+  else String.concat "/" l (* BYXXX : check illicit characters *)
 
 let parse_url =
 
   (*SSS Neturl doesn't recognize http://[2002::1]:80. Workaround: we
     chop the http://host:port part and use Neturl for the rest.
     We do not accept http://login:pwd@host:port (should we?). *)
-  let url_re = Netstring_pcre.regexp "^[Hh][Tt][Tt][Pp][Ss]?://([0-9a-zA-Z.-]+|\\[[0-9A-Fa-f:.]+\\])(:([0-9]+))?(/.*)$" in
+  let url_re = Netstring_pcre.regexp "^([Hh][Tt][Tt][Pp][Ss]?)://([0-9a-zA-Z.-]+|\\[[0-9A-Fa-f:.]+\\])(:([0-9]+))?(/.*)$" in
   let url_relax_re = Netstring_pcre.regexp "^[Hh][Tt][Tt][Pp][Ss]?://[^/]+" in
 
   fun url ->
-    let (host, port, url2) =
+    let (https, host, port, url2) =
       try
         let url2 = Neturl.parse_url
           ~base_syntax:(Hashtbl.find Neturl.common_url_syntax "http")
           (fixup_url_string url)
         in
+        let https = 
+          try (match Neturl.url_scheme url2 with
+                 | "http" -> Some false
+                 | "https" -> Some true
+                 | _ -> None) 
+          with Not_found -> None 
+        in
         let host = try Some (Neturl.url_host url2) with Not_found -> None in
         let port = try Some (Neturl.url_port url2) with Not_found -> None in
-        (host, port, url2)
+        (https, host, port, url2)
       with Neturl.Malformed_URL ->
         match Netstring_pcre.string_match url_re url 0 with
           | None -> raise Neturl.Malformed_URL
           | Some m ->
               let url2 = Neturl.parse_url
                 ~base_syntax:(Hashtbl.find Neturl.common_url_syntax "http")
-                (fixup_url_string (Netstring_pcre.matched_group m 4 url)) in
+                (fixup_url_string (Netstring_pcre.matched_group m 5 url)) in
+              let https =
+                try (match Netstring_pcre.matched_group m 1 url with
+                       | "http" -> Some false
+                       | "https" -> Some true
+                       | _ -> None)
+                with Not_found -> None in
               let host =
-                try Some (Netstring_pcre.matched_group m 1 url)
+                try Some (Netstring_pcre.matched_group m 2 url)
                 with Not_found -> None in
               let port =
-                try Some (int_of_string (Netstring_pcre.matched_group m 3 url))
+                try Some (int_of_string (Netstring_pcre.matched_group m 4 url))
                 with Not_found -> None in
-              (host, port, url2)
+              (https, host, port, url2)
     in
 
     (* We don't do it before because we don't want [] of IPv6
@@ -511,7 +590,22 @@ let parse_url =
            I prefer forbid that. *)
     in
 
-    (host, port, url, url2, path, params, get_params)
+    (https, host, port, url, url2, path, params, get_params)
+
+
+(************************************************************************)
+(* absolute urls *)
+
+let make_absolute_url ~https ~host ~port uri =
+  (if https
+   then "https://"
+   else "http://"
+  )^
+    host^
+    (if (port = 80 && not https) || (https && port = 443)
+     then ""
+     else ":"^string_of_int port)^
+    uri
 
 
 (************************************************************************)
@@ -527,8 +621,31 @@ let basename f =
     "none"
 
 
+let extension_no_directory filename =
+  try
+    let pos = String.rindex filename '.' in
+    String.sub filename (pos+1) ((String.length filename) - pos - 1)
+  with Not_found ->
+    raise Not_found
+
+let extension filename =
+  try
+    let pos = String.rindex filename '.'
+    and slash =
+      try String.rindex filename '/'
+      with Not_found -> -1
+    in
+    if pos > slash then
+      String.sub filename (pos+1) ((String.length filename) - pos - 1)
+    else (* Dot before a directory separator *)
+      raise Not_found
+  with Not_found -> (* No dot in filename *)
+    raise Not_found
+
+
 (* *)
 type ('a, 'b) leftright = Left of 'a | Right of 'b
+type yesnomaybe = Yes | No | Maybe
 
 module StringSet = Set.Make(String)
 
