@@ -23,12 +23,19 @@
 
 (********)
 
+type att_key =
+  | Att_no
+  | Att_named of string (* named *)
+  | Att_anon of string (* anonymous *)
+
 type na_key =
   | Na_no
-  | Na_get_ of string (* service *)
-  | Na_post_ of string (* service *)
-  | Na_get' of string (* coservice *)
-  | Na_post' of string (* coservice *)
+  | Na_void_keep (* void coservice that keeps GET na parameters *)
+  | Na_void_dontkeep (* void coservice that does not keep GET na parameters *)
+  | Na_get_ of string (* named *)
+  | Na_post_ of string (* named *)
+  | Na_get' of string (* anonymous *)
+  | Na_post' of string (* anonymous *)
 
 
 exception Eliom_Wrong_parameter (** Service called with wrong parameter names *)
@@ -56,11 +63,13 @@ let defaultpagename = "./"
    "index" works but one page may have two different URLs *)
 
 let eliom_suffix_name = "__eliom_suffix"
-let eliom_suffix_internal_name = "__eliom_suffix**"
+let eliom_suffix_internal_name = "__(suffix service)__"
 let naservice_num = "__eliom_na__num"
 let naservice_name = "__eliom_na__name"
 let get_state_param_name = "__eliom__"
 let post_state_param_name = "__eliom_p__"
+let get_numstate_param_name = "__eliom_n__"
+let post_numstate_param_name = "__eliom_np__"
 let co_param_prefix = "__co_eliom_"
 let na_co_param_prefix = "__na_eliom_"
 
@@ -83,11 +92,6 @@ let eliom_persistent_cookie_table =
 
 (*****************************************************************************)
 
-(** state is a parameter to differenciate coservices
-    (several instances of the same URL).
- *)
-type internal_state = string
-
 (** Type used for cookies to set.
     The float option is the timestamp for the expiration date.
     The strings are names and values.
@@ -122,8 +126,7 @@ type sess_info =
      (* the same, but for secure cookies, if https *)
 
      si_nonatt_info: na_key;
-     si_state_info: (internal_state option * internal_state option);
-     si_config_file_charset: string;
+     si_state_info: (att_key * att_key);
      si_previous_extension_error: int;
      (* HTTP error code sent by previous extension (default: 404) *)
    }
@@ -298,7 +301,7 @@ type result_to_send =
 
 
 type page_table_key =
-    {key_state: (internal_state option * internal_state option);
+    {key_state : att_key * att_key;
      key_kind: Ocsigen_http_frame.Http_header.http_method}
       (* action: server_params -> page *)
 
@@ -318,7 +321,7 @@ end)
 type anon_params_type = int
 
 type server_params =
-    {sp_ri:Ocsigen_extensions.request_info;
+    {sp_request:Ocsigen_extensions.request;
      sp_si:sess_info;
      sp_sitedata:sitedata (* data for the whole site *);
      sp_cookie_info:tables cookie_info;
@@ -388,9 +391,6 @@ and sitedata =
    mutable max_volatile_data_sessions_per_group: int option;
    mutable max_service_sessions_per_group: int option;
    mutable max_persistent_data_sessions_per_group: int option;
-   defaulthostname: string;
-   defaulthttpport: int;
-   defaulthttpsport: int;
  }
 
 
@@ -398,7 +398,7 @@ and sitedata =
     (** Create server parameters record *)
 let make_server_params sitedata all_cookie_info ri suffix si fullsessname
     : server_params =
-  {sp_ri=ri;
+  {sp_request=ri;
    sp_si=si;
    sp_sitedata=sitedata;
    sp_cookie_info=all_cookie_info;
@@ -457,7 +457,9 @@ let getcookies cookiename cookies =
     Ocsigen_http_frame.Cookievalues.empty
 
 
-let change_request_info ri charset previous_extension_err =
+let change_request_info ri previous_extension_err =
+  let ri_whole = ri
+  and ri = ri.Ocsigen_extensions.request_info in
   Lazy.force ri.Ocsigen_extensions.ri_post_params >>=
   (fun post_params ->
     let get_params = Lazy.force ri.Ocsigen_extensions.ri_get_params in
@@ -514,7 +516,7 @@ let change_request_info ri charset previous_extension_err =
         | Na_post_ _
         | Na_post' _ -> (* POST non attached coservice *)
             (post_naservice_name,
-             (None, None),
+             (Att_no, Att_no),
              ([], get_params),
              na_post_params)
         | _ ->
@@ -536,7 +538,7 @@ let change_request_info ri charset previous_extension_err =
               | Na_get_ _
               | Na_get' _ -> (* GET non attached coservice *)
                   (get_naservice_name,
-                   (None, None),
+                   (Att_no, Att_no),
                    (na_get_params, other_get_params),
                    [])
                     (* Not possible to have POST parameters
@@ -548,19 +550,33 @@ let change_request_info ri charset previous_extension_err =
                     try
                       let s, pp =
                         Ocsigen_lib.list_assoc_remove
-                          post_state_param_name post_params
-                      in (Some s, pp)
+                          post_numstate_param_name post_params
+                      in (Att_anon s, pp)
                     with
-                        Not_found -> (None, post_params)
+                        Not_found -> 
+                          try
+                            let s, pp =
+                              Ocsigen_lib.list_assoc_remove
+                                post_state_param_name post_params
+                            in (Att_named s, pp)
+                          with
+                              Not_found -> (Att_no, post_params)
                   in
                   let get_state, (get_params, other_get_params) =
                     try
                       let s, gp =
                         Ocsigen_lib.list_assoc_remove
-                          get_state_param_name get_params
-                      in ((Some s),
+                          get_numstate_param_name get_params
+                      in ((Att_anon s),
                           (split_prefix_param co_param_prefix gp))
-                    with Not_found -> (None, (get_params, []))
+                    with Not_found ->
+                      try
+                        let s, gp =
+                          Ocsigen_lib.list_assoc_remove
+                            get_state_param_name get_params
+                        in ((Att_named s),
+                            (split_prefix_param co_param_prefix gp))
+                      with Not_found -> (Att_no, (get_params, []))
                   in
                   (Na_no,
                    (get_state, post_state),
@@ -568,8 +584,8 @@ let change_request_info ri charset previous_extension_err =
                    post_params)
     in
 
-    return
-      ({ri with
+    let ri', sess =
+      {ri with
         Ocsigen_extensions.ri_method =
         (if ri.Ocsigen_extensions.ri_method = Ocsigen_http_frame.Http_header.HEAD
         then Ocsigen_http_frame.Http_header.GET
@@ -585,8 +601,11 @@ let change_request_info ri charset previous_extension_err =
         si_other_get_params= other_get_params;
         si_all_get_params= get_params0;
         si_all_post_params= post_params0;
-        si_config_file_charset= charset;
-        si_previous_extension_error= previous_extension_err}))
+        si_previous_extension_error= previous_extension_err}
+    in
+    Lwt.return
+    ({ ri_whole with Ocsigen_extensions.request_info = ri' }, sess))
+
 
 
 
@@ -610,7 +629,7 @@ let make_fullsessname2 site_dir_string = function
 
 (*****************************************************************************)
 exception Eliom_retry_with of
-  (Ocsigen_extensions.request_info *
+  (Ocsigen_extensions.request *
      sess_info *
      Ocsigen_http_frame.cookieset (* user cookies set by previous pages *) *
      tables cookie_info
@@ -638,8 +657,8 @@ let create_persistent_table name =
 let persistent_cookies_table :
     (string * float option * timeout *
        Eliommod_sessiongroups.perssessgrp option)
-    Ocsipersist.table =
-  create_persistent_table eliom_persistent_cookie_table
+    Ocsipersist.table Lazy.t =
+  lazy (create_persistent_table eliom_persistent_cookie_table)
 (* Another tables, containing the session info for each cookie *)
 (* the table contains:
    - the expiration date (by timeout), changed at each access to the table
@@ -647,6 +666,10 @@ let persistent_cookies_table :
    - the timeout for the user (float option option) None -> see global config
      Some None -> no timeout
  *)
+(* It is lazy, because we must delay the creation of the table until
+   the initialization of eliom in case we use static linking with 
+   sqlite backend ... *)
+
 
 (** removes the entry from all opened tables *)
 let remove_from_all_persistent_tables key =
