@@ -249,8 +249,12 @@ module type ELIOMFORMSIG =
 (** Creates the string corresponding to the
     full (absolute) URL of a service applied to its GET parameters.
 
-    Hostname and port are determined from attribute of <host> tag in
-    configuration file, but you can choose the one you want by setting 
+    Default hostname is determined from the [Host] http header of the request
+    (or the attribute of <host> tag in
+    configuration file if the option [<usedefaulthostname/>] is set).
+    Default port is the current port (or another port of the server if
+    you are switching from or to https).
+    But you can choose the hostname or port you want by setting 
     the optional [?hostname] and [?port] parameters here.
  *)
 
@@ -328,13 +332,16 @@ module type ELIOMFORMSIG =
     to make easier the use with reverse proxies.
     But in case of protocol change (if you want to switch to https using
     [~https:true] for example, or if the service imposes https),
-    absolute links will be generated. In that case,
-    hostname and port are determined from attribute of [<host>] tag in
-    configuration file, but you can choose other ones by setting 
+    absolute links will be generated. 
+    In that case,
+    default hostname is determined from the [Host] http header of the request
+    (or the attribute of <host> tag in
+    configuration file if the option [<usedefaulthostname/>] is set).
+    Default port is the current port (or another port of the server if
+    you are switching from or to https).
+    But you can choose the hostname or port you want by setting 
     the optional [?hostname] and [?port] parameters here.
     These options have no effect for relative links.
-    Warning: it may be a security problem to use the hostname sent by the client
-    here.
 
 *)
 
@@ -904,91 +911,102 @@ module MakeForms = functor
               let params_string =
                 concat_strings preapplied_params "&" params_string in
               let uri =
-                (if (get_att_kind_ attser) = `External
-                 then
+                if (get_att_kind_ attser) = `External
+                then
                   (get_prefix_ attser)^
                     "/"^  (* we add the "/" even if there is no prefix,
                              because we should do absolute links in that case *)
                     (reconstruct_absolute_url_path
                        (get_full_path_ attser) suff)
                 else
-                  if absolute
-                  then
-                    reconstruct_absolute_url_path (get_full_path_ attser) suff
-                  else
-                    reconstruct_relative_url_path
-                      (get_current_full_path sp) (get_full_path_ attser) suff)
+                  match absolute with
+                    | Some proto_prefix ->
+                        proto_prefix^
+                          reconstruct_absolute_url_path
+                          (get_full_path_ attser) suff
+                    | None ->
+                        reconstruct_relative_url_path
+                          (get_original_full_path sp)
+                          (get_full_path_ attser) suff
               in
-              match get_get_state_ attser with
-              | None ->
-                  add_to_string
-                    (add_to_string uri "?" params_string)
-                    "#"
-                    (Netencoding.Url.encode fragment)
-              | Some s ->
-                  add_to_string
-                    (add_to_string (uri^"?"^Eliom_common.get_state_param_name^"="^s)
-                       "&" params_string)
-                    "#"
-                    (Netencoding.Url.encode fragment)
+              match get_get_name_ attser with
+                | Eliom_common.Att_no ->
+                    add_to_string
+                      (add_to_string uri "?" params_string)
+                      "#"
+                      (Netencoding.Url.encode fragment)
+                | Eliom_common.Att_anon s ->
+                    add_to_string
+                      (add_to_string
+                         (uri^"?"^Eliom_common.get_numstate_param_name^"="^s)
+                         "&" params_string)
+                      "#"
+                      (Netencoding.Url.encode fragment)
+                | Eliom_common.Att_named s ->
+                    add_to_string
+                      (add_to_string
+                         (uri^"?"^Eliom_common.get_state_param_name^"="^s)
+                         "&" params_string)
+                      "#"
+                      (Netencoding.Url.encode fragment)
             end
         | `Nonattached naser ->
-            let current_get_params =
-              List.remove_assoc
-                Eliom_common.naservice_name
-                (List.remove_assoc
-                   Eliom_common.naservice_num
-                   (remove_prefixed_param
-                      Eliom_common.na_co_param_prefix
-                      (Eliom_sessions.get_all_current_get_params sp)))
-            in
-            let gp =
-              match
-                match get_na_name_ naser with
-                  | Eliom_common.Na_get_ "" -> None
-                      (* The empty name is for the "cancel" service,
-                         that goes back to the URL without na-parameters
-                      *)
-                  | Eliom_common.Na_get' n ->
-                      Some (Eliom_common.naservice_num^"="^n)
-                  | Eliom_common.Na_get_ n ->
-                      Some (Eliom_common.naservice_name^"="^n)
-                  | _ -> assert false
-              with
-                | None -> ""
-                | Some naservice_param ->
-                    let _, params_string =
-                      construct_params
-                        (get_get_params_type_ service)
-                        getparams
-                    in
-                    let preapplied_params =
-                      construct_params_string
-                        (get_pre_applied_parameters_ service)
-                    in
-                    let params_string =
-                      concat_strings preapplied_params "&" params_string
-                    in
-                    (concat_strings naservice_param "&" params_string)
-            in
-            let current_get_params_string =
-              construct_params_string current_get_params
+            let na_name = get_na_name_ naser in
+            let params =
+              if na_name = Eliom_common.Na_void_keep 
+              then construct_params_string (get_initial_get_params sp)
+              else
+                let current_get_params =
+                  List.remove_assoc
+                    Eliom_common.naservice_name
+                    (List.remove_assoc
+                       Eliom_common.naservice_num
+                       (remove_prefixed_param
+                          Eliom_common.na_co_param_prefix
+                          (Eliom_sessions.get_all_current_get_params sp)))
+                in
+                let gp =
+                  match
+                    match na_name with
+                      | Eliom_common.Na_void_dontkeep -> None
+                      | Eliom_common.Na_get' n ->
+                          Some (Eliom_common.naservice_num^"="^n)
+                      | Eliom_common.Na_get_ n ->
+                          Some (Eliom_common.naservice_name^"="^n)
+                      | _ -> assert false
+                  with
+                    | None -> ""
+                    | Some naservice_param ->
+                        let _, params_string =
+                          construct_params
+                            (get_get_params_type_ service)
+                            getparams
+                        in
+                        let preapplied_params =
+                          construct_params_string
+                            (get_pre_applied_parameters_ service)
+                        in
+                        let params_string =
+                          concat_strings preapplied_params "&" params_string
+                        in
+                        (concat_strings naservice_param "&" params_string)
+                in
+                let current_get_params_string =
+                  construct_params_string current_get_params
+                in
+                concat_strings
+                  current_get_params_string
+                  "&"
+                  gp
             in
             let beg =
-              if absolute
-              then get_current_full_path_string sp
-              else relative_url_path_to_myself (get_current_sub_path sp)
+              match absolute with
+                | Some proto_prefix ->
+                    proto_prefix^ get_original_full_path_string sp
+                | None -> 
+                    relative_url_path_to_myself (get_original_full_path sp)
             in
-            let params =
-              concat_strings
-                current_get_params_string
-                "&"
-                gp
-            in
-            if params = ""
-            then beg (* may be empty *)
-            else beg^"?"^params
-
+            add_to_string beg "?" params
 
       let make_proto_prefix
           ~sp
@@ -1011,15 +1029,7 @@ module MakeForms = functor
                 then Eliom_sessions.get_default_sslport ~sp
                 else Eliom_sessions.get_default_port ~sp
         in
-        (if https
-         then "https://"
-         else "http://"
-        )^
-          host^
-          (if (port = 80 && not https) || (https && port = 443)
-           then ""
-           else ":"^string_of_int port)^
-          "/"
+        Ocsigen_lib.make_absolute_url https host port "/"
 
       let make_full_string_uri
           ?https
@@ -1029,14 +1039,16 @@ module MakeForms = functor
           ?port
           ?fragment
           getparams : string =
-        (make_proto_prefix ~sp
-           ?hostname
-           ?port
-           ((https = Some true) || 
-              (Eliom_services.get_https service) ||
-              (https = None && Eliom_sessions.get_ssl ~sp)))
-        ^make_string_uri_
-          true
+        let proto_prefix =
+          make_proto_prefix ~sp
+            ?hostname
+            ?port
+            ((https = Some true) || 
+               (Eliom_services.get_https service) ||
+               (https = None && Eliom_sessions.get_ssl ~sp))
+        in
+        make_string_uri_
+          (Some proto_prefix)
           ~service
           ~sp
           ?fragment
@@ -1065,21 +1077,23 @@ module MakeForms = functor
           ?fragment
           getparams : string =
         let ssl = Eliom_sessions.get_ssl ~sp in
-        match https with
-          | Some true when not ssl ->
+        let https = ((https = Some true) || 
+                       (Eliom_services.get_https service) ||
+                       (https = None && Eliom_sessions.get_ssl ~sp))
+(*VVV test duplicated above in make_full_string_uri *)
+        in
+        if https <> ssl
 (*VVV We trust current protocol? *) 
-              make_full_string_uri ?https ~service ~sp
-                ?hostname ?port ?fragment getparams
-          | Some false when ssl ->
-              make_full_string_uri ?https ~service ~sp
-                ?hostname ?port ?fragment getparams
-          | _ ->
-              make_string_uri_
-                false
-                ~service
-                ~sp
-                ?fragment
-                getparams
+        then
+          make_full_string_uri ~https ~service ~sp
+            ?hostname ?port ?fragment getparams
+        else
+          make_string_uri_
+            None
+            ~service
+            ~sp
+            ?fragment
+            getparams
 
       let a 
           ?https
@@ -1091,116 +1105,11 @@ module MakeForms = functor
           ?(fragment = "")
           content
           getparams =
-        let ssl = Eliom_sessions.get_ssl ~sp in
-        let https = 
-          (https = Some true) || 
-            (Eliom_services.get_https service) ||
-            (https = None && ssl)
+        let href = 
+          make_string_uri 
+            ?https ~service ~sp ?hostname ?port ~fragment getparams
         in
-        let absolute = https <> ssl in
-        let proto_prefix = make_proto_prefix ~sp ?hostname ?port https in
-(*VVV We trust current protocol? *) 
-        match get_kind_ service with
-        | `Attached attser ->
-            (let suff, params_string =
-              construct_params (get_get_params_type_ service) getparams in
-            let preapplied_params =
-              construct_params_string (get_pre_applied_parameters_ service) in
-            let params_string =
-              concat_strings preapplied_params "&" params_string in
-            let uri =
-              if (get_att_kind_ attser) = `External
-              then
-                (get_prefix_ attser)^
-                  "/"^
-                  (reconstruct_absolute_url_path
-                     (get_full_path_ attser) suff)
-              else
-                if absolute
-                then
-                  proto_prefix^
-                    reconstruct_absolute_url_path (get_full_path_ attser) suff
-                else
-                  reconstruct_relative_url_path
-                    (get_current_full_path sp) (get_full_path_ attser) suff
-            in
-            match get_get_state_ attser with
-            | None ->
-                Pages.make_a
-                  ?a
-                  ~href:(add_to_string
-                           (add_to_string uri "?" params_string)
-                           "#"
-                           (Netencoding.Url.encode fragment)
-                        ) content
-            | Some s ->
-                Pages.make_a ?a
-                  ~href:
-                  (add_to_string
-                     (add_to_string
-                        (uri^"?"^Eliom_common.get_state_param_name^"="^s)
-                        "&" params_string)
-                     "#"
-                     (Netencoding.Url.encode fragment))
-                  content)
-        | `Nonattached naser ->
-            let current_get_params =
-              List.remove_assoc
-                Eliom_common.naservice_name
-                (List.remove_assoc
-                   Eliom_common.naservice_num
-                   (remove_prefixed_param
-                      Eliom_common.na_co_param_prefix
-                      (Eliom_sessions.get_all_current_get_params sp)))
-            in
-            let gp =
-              match
-                match get_na_name_ naser with
-                  | Eliom_common.Na_get_ "" -> None
-                      (* The empty name is for the "cancel" service,
-                         that goes back to the URL without na-parameters
-                      *)
-                  | Eliom_common.Na_get' n ->
-                      Some (Eliom_common.naservice_num^"="^n)
-                  | Eliom_common.Na_get_ n ->
-                      Some (Eliom_common.naservice_name^"="^n)
-                  | _ -> assert false
-              with
-                | None -> ""
-                | Some naservice_param ->
-                    let _, params_string =
-                      construct_params
-                        (get_get_params_type_ service)
-                        getparams
-                    in
-                    let preapplied_params =
-                      construct_params_string
-                        (get_pre_applied_parameters_ service)
-                    in
-                    let params_string =
-                      concat_strings preapplied_params "&" params_string
-                    in
-                    (concat_strings naservice_param "&" params_string)
-            in
-            let current_get_params_string =
-              construct_params_string current_get_params
-            in
-            let cur = get_current_sub_path sp in
-            let beg =
-              if absolute
-              then proto_prefix^get_current_full_path_string sp
-              else relative_url_path_to_myself cur
-            in
-            let href =
-              concat_strings
-                beg
-                "?"
-                (concat_strings
-                   current_get_params_string
-                   "&"
-                   gp)
-            in
-            Pages.make_a ?a ~href content
+        Pages.make_a ?a ~href content
 
       let get_form_
           bind
@@ -1219,8 +1128,11 @@ module MakeForms = functor
             (Eliom_services.get_https service) ||
             (https = None && ssl)
         in
-        let absolute = https <> ssl in
-        let proto_prefix = make_proto_prefix ~sp ?hostname ?port https in
+        let absolute = 
+          if https <> ssl 
+          then Some (make_proto_prefix ~sp ?hostname ?port https)
+          else None 
+        in
 (*VVV We trust current protocol? *) 
         match get_kind_ service with
         | `Attached attser ->
@@ -1232,21 +1144,27 @@ module MakeForms = functor
                   (reconstruct_absolute_url_path
                      (get_full_path_ attser) None)
               else 
-                if absolute
-                then
-                  proto_prefix^
-                    reconstruct_absolute_url_path (get_full_path_ attser) None
-                else
-                  reconstruct_relative_url_path
-                    (get_current_full_path sp) (get_full_path_ attser) None
+                match absolute with
+                  | Some proto_prefix ->
+                      proto_prefix^
+                        reconstruct_absolute_url_path
+                        (get_full_path_ attser) None
+                  | None ->
+                      reconstruct_relative_url_path
+                        (get_original_full_path sp)
+                        (get_full_path_ attser) None
             in
             let urlname =
               add_to_string urlname "#" (Netencoding.Url.encode fragment)
             in
             let state_param =
-              (match get_get_state_ attser with
-              | None -> None
-              | Some s ->
+              (match get_get_name_ attser with
+              | Eliom_common.Att_no -> None
+              | Eliom_common.Att_anon s ->
+                  Some (Pages.make_input ~typ:Pages.hidden
+                          ~name:Eliom_common.get_numstate_param_name
+                          ~value:s ())
+              | Eliom_common.Att_named s ->
                   Some (Pages.make_input ~typ:Pages.hidden
                           ~name:Eliom_common.get_state_param_name
                           ~value:s ()))
@@ -1273,74 +1191,96 @@ module MakeForms = functor
                in 
                return (Pages.make_get_form ?a ~action:urlname i1 i))
         | `Nonattached naser ->
-            let cur = get_current_sub_path sp in
-            let urlname =
-              if absolute
-              then proto_prefix^get_current_full_path_string sp
-              else relative_url_path_to_myself cur
-            in
-            (* "/"^(get_current_path_string sp) --> absolute (wrong) *)
-            let current_get_params =
-              List.remove_assoc
-                Eliom_common.naservice_name
-                (List.remove_assoc
-                   Eliom_common.naservice_num
-                   (remove_prefixed_param
-                      Eliom_common.na_co_param_prefix
-                      (get_all_current_get_params sp)))
-            in
-            let naservice_line =
-              match get_na_name_ naser with
-                | Eliom_common.Na_get_ "" ->
-                    Pages.make_hidden_field None
-                | Eliom_common.Na_get' n ->
-                    Pages.make_hidden_field
-                      (Some (Pages.make_input
-                               ~typ:Pages.hidden
-                               ~name:Eliom_common.naservice_num
-                               ~value:n ()))
-                | Eliom_common.Na_get_ n ->
-                    Pages.make_hidden_field
-                      (Some (Pages.make_input
-                               ~typ:Pages.hidden
-                               ~name:Eliom_common.naservice_name
-                               ~value:n ()))
-                | _ -> assert false
-            in
-            bind (f (make_params_names (get_get_params_type_ service)))
-            (fun inside ->
-               let all_lines =
-                 List.fold_left
-                   (fun s (n,v) ->
-                      Pages.cons_form
-                        (Pages.make_hidden_field
-                           (Some
-                              (Pages.make_input
+            let na_name = get_na_name_ naser in
+            if na_name = Eliom_common.Na_void_keep 
+            then (* void coservice' *)
+              let params = 
+                construct_params_string (get_initial_get_params sp)
+              in
+              let href = 
+                let beg =
+                  match absolute with
+                    | Some proto_prefix ->
+                        proto_prefix^ get_original_full_path_string sp
+                    | None -> 
+                        relative_url_path_to_myself (get_original_full_path sp)
+                in
+                add_to_string beg "?" params
+              in
+              bind (f (make_params_names (get_get_params_type_ service)))
+                (fun inside ->
+                   let i1, i = Pages.remove_first inside in 
+                   return (Pages.make_get_form ?a ~action:href i1 i))
+
+            else
+              let urlname =
+                match absolute with
+                  | Some proto_prefix ->
+                      proto_prefix^get_original_full_path_string sp
+                  | None ->
+                      relative_url_path_to_myself (get_original_full_path sp)
+              in
+              let current_get_params =
+                List.remove_assoc
+                  Eliom_common.naservice_name
+                  (List.remove_assoc
+                     Eliom_common.naservice_num
+                     (remove_prefixed_param
+                        Eliom_common.na_co_param_prefix
+                        (get_all_current_get_params sp)))
+              in
+              let naservice_line =
+                match na_name with
+                  | Eliom_common.Na_void_dontkeep ->
+                      Pages.make_hidden_field None
+                  | Eliom_common.Na_get' n ->
+                      Pages.make_hidden_field
+                        (Some (Pages.make_input
                                  ~typ:Pages.hidden
-                                 ~name:n ~value:v ())))
-                        s
-                   )
-                   inside
-                   current_get_params
-               in
-               let all_lines =
-                 List.fold_left
-                   (fun s (n,v) ->
-                      Pages.cons_form
-                        (Pages.make_hidden_field
-                           (Some
-                              (Pages.make_input
+                                 ~name:Eliom_common.naservice_num
+                                 ~value:n ()))
+                  | Eliom_common.Na_get_ n ->
+                      Pages.make_hidden_field
+                        (Some (Pages.make_input
                                  ~typ:Pages.hidden
-                                 ~name:n
-                                 ~value:v ())))
-                        s
-                   )
-                   all_lines
-                   (get_pre_applied_parameters_ service)
-               in
-               return
-                 (Pages.make_get_form
-                    ?a ~action:urlname naservice_line all_lines))
+                                 ~name:Eliom_common.naservice_name
+                                 ~value:n ()))
+                  | _ -> assert false
+              in
+              bind (f (make_params_names (get_get_params_type_ service)))
+                (fun inside ->
+                   let all_lines =
+                     List.fold_left
+                       (fun s (n,v) ->
+                          Pages.cons_form
+                            (Pages.make_hidden_field
+                               (Some
+                                  (Pages.make_input
+                                     ~typ:Pages.hidden
+                                     ~name:n ~value:v ())))
+                            s
+                       )
+                       inside
+                       current_get_params
+                   in
+                   let all_lines =
+                     List.fold_left
+                       (fun s (n,v) ->
+                          Pages.cons_form
+                            (Pages.make_hidden_field
+                               (Some
+                                  (Pages.make_input
+                                     ~typ:Pages.hidden
+                                     ~name:n
+                                     ~value:v ())))
+                            s
+                       )
+                       all_lines
+                       (get_pre_applied_parameters_ service)
+                   in
+                   return
+                     (Pages.make_get_form
+                        ?a ~action:urlname naservice_line all_lines))
 
       let get_form ?https ?a ~service ~sp ?hostname ?port ?fragment f =
         get_form_ (fun x f -> f x) (fun x -> x) 
@@ -1383,9 +1323,14 @@ module MakeForms = functor
             let params_string =
               concat_strings preapplied_params "&" params_string in
             let params_string =
-              match get_get_state_ attser with
-              | None -> params_string
-              | Some s ->
+              match get_get_name_ attser with
+              | Eliom_common.Att_no -> params_string
+              | Eliom_common.Att_anon s ->
+                  add_to_string
+                    (Eliom_common.get_numstate_param_name^"="^s)
+                    "&"
+                    params_string
+              | Eliom_common.Att_named s ->
                   add_to_string
                     (Eliom_common.get_state_param_name^"="^s)
                     "&"
@@ -1405,15 +1350,19 @@ module MakeForms = functor
                     reconstruct_absolute_url_path (get_full_path_ attser) suff
                 else
                   reconstruct_relative_url_path
-                    (get_current_full_path sp) (get_full_path_ attser) suff
+                    (get_original_full_path sp) (get_full_path_ attser) suff
             in
             let urlname =
               add_to_string urlname "#" (Netencoding.Url.encode fragment)
             in
             let state_param =
-              (match get_post_state_ attser with
-              | None -> None
-              | Some s ->
+              (match get_post_name_ attser with
+              | Eliom_common.Att_no -> None
+              | Eliom_common.Att_anon s ->
+                  Some (Pages.make_input ~typ:Pages.hidden
+                          ~name:Eliom_common.post_numstate_param_name
+                          ~value:s ())
+              | Eliom_common.Att_named s ->
                   Some (Pages.make_input ~typ:Pages.hidden
                           ~name:Eliom_common.post_state_param_name
                           ~value:s ()))
@@ -1454,12 +1403,11 @@ module MakeForms = functor
             let current_get_params_string =
               construct_params_string current_get_params
             in
-            let cur = get_current_sub_path sp in
             (* absolute URL does not work behind a revproxy! *)
             let urlpath =
               if absolute
-              then proto_prefix^get_current_full_path_string sp
-              else relative_url_path_to_myself cur
+              then proto_prefix^get_original_full_path_string sp
+              else relative_url_path_to_myself (get_original_full_path sp)
             in
             let v = concat_strings urlpath "?" current_get_params_string in
             let naservice_line =
