@@ -69,9 +69,14 @@ module type XHTMLFORMSSIG = sig
 (** Creates the string corresponding to the
     full (absolute) URL of a service applied to its GET parameters.
 
-    Hostname and port are determined from attribute of <host> tag in
-    configuration file, but you can choose the one you want by setting 
+    Default hostname is determined from the [Host] http header of the request
+    (or the attribute of <host> tag in
+    configuration file if the option [<usedefaulthostname/>] is set).
+    Default port is the current port (or another port of the server if
+    you are switching from or to https).
+    But you can choose the hostname or port you want by setting 
     the optional [?hostname] and [?port] parameters here.
+
  *)
 
     val make_full_uri :
@@ -152,13 +157,16 @@ module type XHTMLFORMSSIG = sig
     to make easier the use with reverse proxies.
     But in case of protocol change (if you want to switch to https using
     [~https:true] for example, or if the service imposes https),
-    absolute links will be generated. In that case,
-    hostname and port are determined from attribute of [<host>] tag in
-    configuration file, but you can choose other ones by setting 
+    absolute links will be generated. 
+    In that case,
+    default hostname is determined from the [Host] http header of the request
+    (or the attribute of <host> tag in
+    configuration file if the option [<usedefaulthostname/>] is set).
+    Default port is the current port (or another port of the server if
+    you are switching from or to https).
+    But you can choose the hostname or port you want by setting 
     the optional [?hostname] and [?port] parameters here.
     These options have no effect for relative links.
-    Warning: it may be a security problem to use the hostname sent by the client
-    here.
 
  *)
 
@@ -689,20 +697,40 @@ end
       (** Eliom forms and service registration functions for XHTML *)
 module Xhtml : sig
 
-  include Eliom_mkreg.ELIOMREGSIG with type page = xhtml elt
+  include Eliom_mkreg.ELIOMREGSIG with type page = xhtml elt 
+                                  and type options = XHTML.M.doctypes
+
   include XHTMLFORMSSIG
 
 end
 
+module Xhtmlforms : XHTMLFORMSSIG
+module Xhtmlreg : Eliom_mkreg.ELIOMREGSIG with type page = xhtml elt 
+                                  and type options = XHTML.M.doctypes
+
+module Xhtmlreg_ : 
+  functor(Xhtml_content : 
+            Ocsigen_http_frame.HTTP_CONTENT with type t = [ `Html ] XHTML.M.elt
+                                            and type options = XHTML.M.doctypes
+         ) -> Eliom_mkreg.REGCREATE with type page =  Xhtml_content.t
+                                    and type options = XHTML.M.doctypes
+
+
+
       (** Eliom forms and service registration functions for XHTML, with
          compact markup (i.e., without pretty-printing). *)
 module Xhtmlcompact : sig
 
   include Eliom_mkreg.ELIOMREGSIG with type page = xhtml elt
+                                  and type options = XHTML.M.doctypes
+
   include XHTMLFORMSSIG
 
 end
 
+module Xhtmlcompactreg : Eliom_mkreg.ELIOMREGSIG with type page = xhtml elt 
+                                  and type options = XHTML.M.doctypes
+
 (** {3 Module to register subpages of type [block]} *)
 
 module Blocks : sig
@@ -802,7 +830,7 @@ module Unit : Eliom_mkreg.ELIOMREGSIG with
    To choose if you want permanent or temporary redirection, use
    the [options] parameter of registration functions.
    For example: [register ~options:`Temporary ...].
- *)
+*)
 module Redirection : Eliom_mkreg.ELIOMREGSIG with
   type page =
   (unit, unit, Eliom_services.get_service_kind,
@@ -823,13 +851,14 @@ module Redirection : Eliom_mkreg.ELIOMREGSIG with
  *)
 module String_redirection : Eliom_mkreg.ELIOMREGSIG with
   type page = XHTML.M.uri
-(*VVV Would be better to define the type uri elsewhere *)
   and type options = [ `Temporary | `Permanent ]
+(*VVV Would be better to define the type uri elsewhere *)
 
 (** Allows to send files. The content is the name of the file to send. *)
 module Files : Eliom_mkreg.ELIOMREGSIG with
   type page = string
 
+
 (** Allows to create services that choose dynamically what they want
    to send. The content is created using one {!Eliom_mkreg.ELIOMREGSIG1.send}
    function, for ex [Xhtml.send] or [Files.send].
