@@ -94,8 +94,8 @@ type +'a a_s =
      subpath: url_path; (* name of the service without parameters *)
      fullpath: url_path; (* full path of the service = site_dir@subpath *)
      att_kind: 'a; (* < attached_service_kind *)
-     get_state: Eliom_common.internal_state option;
-     post_state: Eliom_common.internal_state option;
+     get_name: Eliom_common.att_key;
+     post_name: Eliom_common.att_key;
    }
 
 type +'a na_s =
@@ -151,8 +151,8 @@ let get_post_params_type_ s = s.post_params_type
 let get_prefix_ s = s.prefix
 let get_sub_path_ s = s.subpath
 let get_full_path_ s = s.fullpath
-let get_get_state_ s = s.get_state
-let get_post_state_ s = s.post_state
+let get_get_name_ s = s.get_name
+let get_post_name_ s = s.post_name
 let get_na_name_ s = s.na_name
 let get_na_kind_ s = s.na_kind
 let get_max_use_ s = s.max_use
@@ -169,6 +169,14 @@ let new_state =
     (Printf.sprintf "%x" (Random.int 0xFFFF))^(Printf.sprintf "%Lx" !c)
 
 
+(*****************************************************************************)
+(*****************************************************************************)
+(* Registration of static module initialization functions                    *)
+(*****************************************************************************)
+(*****************************************************************************)
+
+let register_eliom_module name f =
+  Ocsigen_loader.set_module_init_function name f
 
 (*****************************************************************************)
 (*****************************************************************************)
@@ -188,8 +196,8 @@ let static_dir_ ?(https = false) ~sp () =
        {prefix = "";
         subpath = [""];
         fullpath = (Eliom_sessions.get_site_dir sp) @ [""];
-        get_state = None;
-        post_state = None;
+        get_name = Eliom_common.Att_no;
+        post_name = Eliom_common.Att_no;
         att_kind = `Internal (`Service, `Get);
       };
      https = https;
@@ -213,8 +221,8 @@ let get_static_dir_ ?(https = false) ~sp ~get_params () =
        {prefix = "";
         subpath = [""];
         fullpath = (Eliom_sessions.get_site_dir sp) @ [""];
-        get_state = None;
-        post_state = None;
+        get_name = Eliom_common.Att_no;
+        post_name = Eliom_common.Att_no;
         att_kind = `Internal (`Service, `Get);
       };
      https = https;
@@ -252,8 +260,8 @@ let new_service_aux_aux
       subpath = path;
       fullpath = site_dir @ path;
       att_kind = kind;
-      get_state = None;
-      post_state = None;
+      get_name = Eliom_common.Att_no;
+      post_name = Eliom_common.Att_no;
     };
    https = https;
  }
@@ -340,6 +348,7 @@ let new_service
 let new_naservice_num () = new_state ()
 
 let new_coservice
+    ?name
     ?max_use
     ?timeout
     ?(https = false)
@@ -356,77 +365,47 @@ let new_coservice
    get_params_type = add_pref_params Eliom_common.co_param_prefix get_params;
    kind = `Attached
      {k with
-      get_state = Some (new_state ());
-      att_kind = `Internal (`Coservice, `Get);
-    };
+      get_name = 
+         (match name with
+           | None -> Eliom_common.Att_anon (new_state ())
+           | Some name -> Eliom_common.Att_named name);
+        att_kind = `Internal (`Coservice, `Get);
+     };
    https = https || fallback.https
  }
 (* Warning: here no GET parameters for the fallback.
-   Apply services with apply_service
-   if you want fallbacks with GET parameters *)
+   Preapply services if you want fallbacks with GET parameters *)
 
 
-let new_coservice' ?max_use ?timeout ?(https = false) ~get_params () =
-  let n = Eliom_common.Na_get' (new_naservice_num ()) in
+let new_coservice' ?name ?max_use ?timeout ?(https = false) ~get_params () =
   (* (match Eliom_common.global_register_allowed () with
   | Some _ -> Eliom_common.add_unregistered_na n;
   | _ -> () (* Do we accept unregistered non-attached coservices? *)); *)
-  {
-   max_use= max_use;
-   timeout= timeout;
-   pre_applied_parameters = [];
-   get_params_type = add_pref_params Eliom_common.na_co_param_prefix get_params;
-   post_params_type = unit;
-   kind = `Nonattached
-     {na_name = n;
-      na_kind = `Get;
-    };
-   https = https;
- }
+  (* (* Do we accept unregistered non-attached named coservices? *)
+     match sp with
+     | None ->
+     ...
+  *)
+        {
+(*VVV allow timeout and max_use for named coservices? *)
+          max_use= max_use;
+          timeout= timeout;
+          pre_applied_parameters = [];
+          get_params_type = 
+            add_pref_params Eliom_common.na_co_param_prefix get_params;
+          post_params_type = unit;
+          kind = `Nonattached
+            {na_name = 
+                match name with
+                  | None ->
+                      Eliom_common.Na_get' (new_naservice_num ())
+                  | Some name -> Eliom_common.Na_get_ name;
+            ;
+             na_kind = `Get;
+            };
+          https = https;
+        }
 
-let new_service' ?sp ?(https = false) ~name ~get_params () =
-  match sp with
-  | None ->
-      (match Eliom_common.global_register_allowed () with
-      | Some get_current_sitedata ->
-          let sitedata = get_current_sitedata () in
-          let r =
-            {
-(*VVV allow timeout and max_use? *)
-             max_use= None;
-             timeout= None;
-             pre_applied_parameters = [];
-             get_params_type =
-             add_pref_params Eliom_common.na_co_param_prefix get_params;
-             post_params_type = unit;
-             kind = `Nonattached
-               {na_name = Eliom_common.Na_get_ name;
-                na_kind = `Get;
-              };
-             https = https;
-           }
-          in
-          Eliom_common.add_unregistered_na sitedata
-            (Eliom_common.Na_get_ name);
-          r
-      | None ->
-          raise (Eliom_common.Eliom_function_forbidden_outside_site_loading
-                   "new_service'"))
-  | Some sp ->
-      {
-(*VVV allow timeout and max_use? *)
-       max_use= None;
-       timeout= None;
-       pre_applied_parameters = [];
-       get_params_type =
-       add_pref_params Eliom_common.na_co_param_prefix get_params;
-       post_params_type = unit;
-       kind = `Nonattached
-         {na_name = Eliom_common.Na_get_ name;
-          na_kind = `Get;
-        };
-       https = https;
-     }
 
 (****************************************************************************)
 (** Register a service with post parameters in the server *)
@@ -446,8 +425,8 @@ let new_post_service_aux ~sp ~https ~fallback ~post_params =
       subpath = k1.subpath;
       fullpath = k1.fullpath;
       att_kind = `Internal (k, `Post);
-      get_state = k1.get_state;
-      post_state = None;
+      get_name = k1.get_name;
+      post_name = Eliom_common.Att_no;
     };
    https = https;
  }
@@ -482,7 +461,7 @@ let new_post_service ?sp ?(https = false) ~fallback ~post_params () =
 
 
 let new_post_coservice 
-    ?max_use ?timeout ?(https = false) ~fallback ~post_params () =
+    ?name ?max_use ?timeout ?(https = false) ~fallback ~post_params () =
   let `Attached k1 = fallback.kind in
   (* (match Eliom_common.global_register_allowed () with
   | Some _ -> Eliom_common.add_unregistered k1.path;
@@ -494,7 +473,10 @@ let new_post_coservice
    kind = `Attached
      {k1 with
       att_kind = `Internal (`Coservice, `Post);
-      post_state = Some (new_state ());
+      post_name = 
+         (match name with
+            | None -> Eliom_common.Att_anon (new_state ())
+            | Some name -> Eliom_common.Att_named name);
     };
    https = https;
  }
@@ -507,42 +489,30 @@ let new_post_coservice
 
 (*VVV Warning: keep_get_na_params is experimental *)
 let new_post_coservice'
+    ?name
     ?max_use ?timeout ?(keep_get_na_params = true) ?(https = false)
     ~post_params () =
   (* match Eliom_common.global_register_allowed () with
   | Some _ -> Eliom_common.add_unregistered None
   | _ -> () *)
   {
-   max_use= max_use;
-   timeout= timeout;
-   pre_applied_parameters = [];
-   get_params_type = unit;
-   post_params_type = post_params;
-   kind = `Nonattached
-     {na_name = Eliom_common.Na_post' (new_naservice_num ());
-      na_kind = `Post keep_get_na_params;
-    };
-   https = https;
- }
+(*VVV allow timeout and max_use for named coservices? *)
+    max_use= max_use;
+    timeout= timeout;
+    pre_applied_parameters = [];
+    get_params_type = unit;
+    post_params_type = post_params;
+    kind = `Nonattached
+      {na_name = 
+          (match name with
+            | None ->
+                Eliom_common.Na_post' (new_naservice_num ())
+            | Some name -> Eliom_common.Na_post_ name);
+       na_kind = `Post keep_get_na_params;
+      };
+    https = https;
+  }
 
-let new_post_service'
-   ?(keep_get_na_params = true) ?(https = false) ~name ~post_params () =
-  (* match Eliom_common.global_register_allowed () with
-  | Some _ -> Eliom_common.add_unregistered None
-  | _ -> () *)
-  {
-(*VVV allow timeout and max_use? *)
-   max_use= None;
-   timeout= None;
-   pre_applied_parameters = [];
-   get_params_type = unit;
-   post_params_type = post_params;
-   kind = `Nonattached
-     {na_name = Eliom_common.Na_post_ name;
-      na_kind = `Post keep_get_na_params;
-    };
-   https = https;
- }
 
 (*
 let new_get_post_coservice'
@@ -577,6 +547,11 @@ Forms towards that kind of service are not implemented
 *)
 
 
+let rec append_suffix l m = match l with
+  | [] -> m
+  | [eliom_suffix_internal_name] -> m
+  | a::ll -> a::(append_suffix ll m)
+
 let preapply ~service getparams =
   let suff, params = construct_params_list service.get_params_type getparams in
   {service with
@@ -585,17 +560,17 @@ let preapply ~service getparams =
    kind = match service.kind with
    | `Attached k -> `Attached {k with
                                subpath = (match suff with
-                               | Some suff -> k.subpath@suff
+                               | Some suff -> append_suffix k.subpath suff
                                | _ -> k.subpath);
                                fullpath = (match suff with
-                               | Some suff -> k.fullpath@suff
+                               | Some suff -> append_suffix k.fullpath suff
                                | _ -> k.fullpath);
                              }
    | k -> k
  }
 
 
-let void_action =
+let void_coservice' =
   {
     max_use= None;
     timeout= None;
@@ -603,13 +578,13 @@ let void_action =
     get_params_type = unit;
     post_params_type = unit;
     kind = `Nonattached
-      {na_name = Eliom_common.Na_get_ "";
+      {na_name = Eliom_common.Na_void_dontkeep;
        na_kind = `Get;
       };
     https = false;
   }
 
-let https_void_action =
+let https_void_coservice' =
   {
     max_use= None;
     timeout= None;
@@ -617,12 +592,26 @@ let https_void_action =
     get_params_type = unit;
     post_params_type = unit;
     kind = `Nonattached
-      {na_name = Eliom_common.Na_get_ "";
+      {na_name = Eliom_common.Na_void_dontkeep;
        na_kind = `Get;
       };
     https = true;
   }
 
+let void_hidden_coservice' = {void_coservice' with 
+                         kind = `Nonattached
+    {na_name = Eliom_common.Na_void_keep;
+     na_kind = `Get;
+    };
+                      }
+
+let https_void_hidden_coservice' = {void_coservice' with 
+                         kind = `Nonattached
+    {na_name = Eliom_common.Na_void_keep;
+     na_kind = `Get;
+    };
+                      }
+
 
 (*****************************************************************************)
 let set_exn_handler ?sp h =
