@@ -20,6 +20,7 @@
  *)
 
 open Lwt
+open Ocsigen_extensions
 
 (*****************************************************************************)
 (* Making accessible some types from Eliom_common: *)
@@ -47,45 +48,51 @@ let find_sitedata fun_name = function
 
 
 let get_user_agent ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_user_agent
+  sp.Eliom_common.sp_request.request_info.ri_user_agent
 let get_full_url ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_url_string
+  sp.Eliom_common.sp_request.request_info.ri_url_string
 let get_remote_ip ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_remote_ip
+  sp.Eliom_common.sp_request.request_info.ri_remote_ip
 let get_remote_inet_addr ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_remote_inet_addr
+  sp.Eliom_common.sp_request.request_info.ri_remote_inet_addr
 let get_get_params ~sp =
-  Lazy.force sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_get_params
+  Lazy.force sp.Eliom_common.sp_request.request_info.ri_get_params
 let get_all_current_get_params ~sp =
   sp.Eliom_common.sp_si.Eliom_common.si_all_get_params
 let get_initial_get_params ~sp =
-  Lazy.force sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_initial_get_params
+  Lazy.force sp.Eliom_common.sp_request.request_info.ri_initial_get_params
 let get_get_params_string ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_get_params_string
+  sp.Eliom_common.sp_request.request_info.ri_get_params_string
 let get_post_params ~sp =
-  Lazy.force sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_post_params
+  Lazy.force sp.Eliom_common.sp_request.request_info.ri_post_params
 let get_all_post_params ~sp =
   sp.Eliom_common.sp_si.Eliom_common.si_all_post_params
+let get_original_full_path_string ~sp =
+  sp.Eliom_common.sp_request.request_info.ri_original_full_path_string
+let get_original_full_path ~sp =
+  sp.Eliom_common.sp_request.request_info.ri_original_full_path
 let get_current_full_path ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_full_path
+  sp.Eliom_common.sp_request.request_info.ri_full_path
 let get_current_full_path_string ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_full_path_string
+  sp.Eliom_common.sp_request.request_info.ri_full_path_string
 let get_current_sub_path ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_sub_path
+  sp.Eliom_common.sp_request.request_info.ri_sub_path
 let get_current_sub_path_string ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_sub_path_string
+  sp.Eliom_common.sp_request.request_info.ri_sub_path_string
 let get_header_hostname ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_host
+  sp.Eliom_common.sp_request.request_info.ri_host
+let get_default_hostname ~sp =
+  sp.Eliom_common.sp_request.request_config.default_hostname
 let get_hostname ~sp =
-  sp.Eliom_common.sp_sitedata.Eliom_common.defaulthostname
+  Ocsigen_extensions.get_hostname sp.Eliom_common.sp_request
 let get_default_port ~sp =
-  sp.Eliom_common.sp_sitedata.Eliom_common.defaulthttpport
+  sp.Eliom_common.sp_request.request_config.default_httpport
 let get_default_sslport ~sp =
-  sp.Eliom_common.sp_sitedata.Eliom_common.defaulthttpsport
+  sp.Eliom_common.sp_request.request_config.default_httpsport
 let get_server_port ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_server_port
+  Ocsigen_extensions.get_port sp.Eliom_common.sp_request
 let get_ssl ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_ssl
+  sp.Eliom_common.sp_request.request_info.ri_ssl
 let get_other_get_params ~sp =
   sp.Eliom_common.sp_si.Eliom_common.si_other_get_params
 let get_suffix ~sp =
@@ -93,11 +100,12 @@ let get_suffix ~sp =
 let get_session_name ~sp =
   sp.Eliom_common.sp_fullsessname
 let get_exn ~sp =
-  sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_extension_info
-let get_config_file_charset ~sp =
-  sp.Eliom_common.sp_si.Eliom_common.si_config_file_charset
+  sp.Eliom_common.sp_request.request_info.ri_extension_info
+let get_config_default_charset ~sp =
+  Ocsigen_charset_mime.default_charset
+    sp.Eliom_common.sp_request.request_config.charset_assoc
 let get_cookies ~sp =
-  Lazy.force sp.Eliom_common.sp_ri.Ocsigen_extensions.ri_cookies
+  Lazy.force sp.Eliom_common.sp_request.request_info.ri_cookies
 let get_data_cookies ~sp =
   sp.Eliom_common.sp_si.Eliom_common.si_data_session_cookies
 let get_persistent_cookies ~sp =
@@ -527,7 +535,9 @@ let get_persistent_data_session_cookie_exp_date ?session_name ?secure ~sp () =
 let get_site_dir ~sp = sp.Eliom_common.sp_sitedata.Eliom_common.site_dir
 let get_site_dir_string ~sp =
   sp.Eliom_common.sp_sitedata.Eliom_common.site_dir_string
-let get_ri ~sp = sp.Eliom_common.sp_ri
+let get_request ~sp = sp.Eliom_common.sp_request
+let get_ri ~sp = sp.Eliom_common.sp_request.Ocsigen_extensions.request_info
+let get_config_info ~sp = sp.Eliom_common.sp_request.Ocsigen_extensions.request_config
 
 let get_tmp_filename fi = fi.Ocsigen_extensions.tmp_filename
 let get_filesize fi = fi.Ocsigen_extensions.filesize
@@ -806,7 +816,7 @@ module Session_admin = struct
     | Some t -> TSome t
     in
     Ocsipersist.add
-      Eliom_common.persistent_cookies_table
+      (Lazy.force Eliom_common.persistent_cookies_table)
       cookie
       (fullsessname, exp, ti, sessgrp)
 
@@ -829,7 +839,7 @@ module Session_admin = struct
   let unset_persistent_data_session_timeout
       ~session:(cookie, (fullsessname, exp, _, sessgrp)) =
     Ocsipersist.add
-      Eliom_common.persistent_cookies_table
+      (Lazy.force Eliom_common.persistent_cookies_table)
       cookie
       (fullsessname, exp, TGlobal, sessgrp)
 
