@@ -82,6 +82,12 @@ val get_current_full_path_string : sp:server_params -> string
 (** returns the full path of the URL using the type {!Ocsigen_extensions.url_path} *)
 val get_current_full_path : sp:server_params -> url_path
 
+(** returns the full path of the URL as first sent by the browser (not changed by previous extensions like rewritemod) *)
+val get_original_full_path_string : sp:server_params -> string
+
+(** returns the full path of the URL as first sent by the browser (not changed by previous extensions like rewritemod) *)
+val get_original_full_path : sp:server_params -> url_path
+
 (** returns the sub path of the URL as a string.
     The sub-path is the full path without the path of the site (set in the
     configuration file).
@@ -99,7 +105,15 @@ val get_current_sub_path : sp:server_params -> url_path
  *)
 val get_header_hostname : sp:server_params -> string option
 
-(** returns the hostname declared in the config file ([<host defaulthostname="...">]).
+(** returns the hostname declared in the config file 
+    ([<host defaulthostname="...">]).
+ *)
+val get_default_hostname : sp:server_params -> string
+
+(** returns the hostname used for absolute links.
+    It is either the [Host] header sent by the browser or the default hostname
+    set in the configuration file, depending on server configuration
+    ([<usedefaulthostname/>] option).
  *)
 val get_hostname : sp:server_params -> string
 
@@ -111,7 +125,11 @@ val get_default_port : sp:server_params -> int
  *)
 val get_default_sslport : sp:server_params -> int
 
-(** returns the port on which the request has been done. *)
+(** returns the port of the server. 
+    It is either the default port in the configuration file,
+    or the port in the Host header of the request,
+    or the port on which the request has been done.
+*)
 val get_server_port : sp:server_params -> int
 
 (** returns true if https is used, false if http. *)
@@ -526,18 +544,20 @@ val get_original_filename : file_info -> string
    (between [<site>] and [</site>]).
 
    {e Warning: You must call that function during the initialisation of
-   your module (not during a Lwt thread or a service).
-   If you use that function after,
-   you must give the [~sp] parameter, otherwise it will raise the exception
-   {!Eliom_common.Eliom_function_forbidden_outside_site_loading}.}
+   your module (not during a Lwt thread or a service)
+   otherwise it will raise the exception
+   {!Eliom_common.Eliom_function_forbidden_outside_site_loading}.
+   If you want to build a statically linkable module, you must call this
+   function inside the initialisation function given to
+   {!Eliom_services.register_eliom_module}.}
  *)
 val get_config : unit -> Simplexmlparser.xml list
 
 (** returns the root of the site. *)
 val get_site_dir : sp:server_params -> url_path
 
-(** returns the charset for this site (from the configuration file) *)
-val get_config_file_charset : sp:server_params -> string
+(** returns the default charset for this site *)
+val get_config_default_charset : sp:server_params -> string
 
 
 
@@ -1013,6 +1033,12 @@ val set_default_persistent_data_session_timeout : float option -> unit
 (** returns all the information about the request. *)
 val get_ri : sp:server_params -> request_info
 
+(** returns information from the configuration files. *)
+val get_config_info : sp:server_params -> config_info
+
+(** returns all the information about the request and config. *)
+val get_request : sp:server_params -> request
+
 (** returns the name of the sessions to which belongs the running service
     ([None] if it is not a session service)
  *)
@@ -1111,5 +1137,10 @@ val set_site_handler : Eliom_common.sitedata ->
   (server_params -> exn -> Eliom_common.result_to_send Lwt.t) -> unit
 
 
+(** Returns the http error code of the request before Eliom was called *)
+val get_previous_extension_error_code :sp:server_params -> int
+
+
+
 val sp_of_esp : Eliom_common.server_params -> server_params
 val esp_of_sp : server_params -> Eliom_common.server_params
