@@ -30,6 +30,30 @@ open Eliom_parameters
 open Eliom_sessions
 open Lwt
 
+(* Suffix and other service at same URL *)
+let su2 =
+  register_new_service
+    ~path:["fuffix";""]
+    ~get_params:(suffix (all_suffix "s"))
+    (fun _ s () ->
+      return
+        (html
+          (head (title (pcdata "")) [])
+          (body [h1
+                   [pcdata 
+                      (Ocsigen_extensions.string_of_url_path
+                         ~encode:false s)];
+                 p [pcdata "Try page fuffix/a"]])))
+
+let su =
+  register_new_service
+    ~path:["fuffix";"a";"b"]
+    ~get_params:unit
+    (fun _ () () ->
+      return
+        (html
+          (head (title (pcdata "")) [])
+          (body [h1 [pcdata "Try another suffix"]])))
 
 
 (* optional parameters *)
@@ -418,7 +442,7 @@ let sendfile2 =
     ~path:["files";""]
     ~get_params:(suffix (all_suffix "filename"))
     (fun _ s () ->
-      return ("/var/www/ocsigen/"^(Ocsigen_extensions.string_of_url_path s)))
+      return ("/var/www/ocsigen/"^(Ocsigen_extensions.string_of_url_path ~encode:false s)))
 
 let sendfileexception =
   register_new_service
@@ -443,7 +467,7 @@ let suffix2 =
            (body
               [p [pcdata "The suffix of the url is ";
                   strong [pcdata (suf1^", "^(string_of_int ii)^", "^
-                                  (Ocsigen_extensions.string_of_url_path ee))]]])))
+                                  (Ocsigen_extensions.string_of_url_path ~encode:false ee))]]])))
 
 let suffix3 =
   register_new_service
@@ -464,7 +488,7 @@ let create_suffixform2 (suf1,(ii,ee)) =
       $string_input ~input_type:`Text ~name:suf1 ()$ <br/>
       Write an int: $int_input ~input_type:`Text ~name:ii ()$ <br/>
       Write a string: $user_type_input ~input_type:`Text ~name:ee
-                         Ocsigen_extensions.string_of_url_path$ <br/>
+                         (Ocsigen_extensions.string_of_url_path ~encode:false)$ <br/>
       $string_input ~input_type:`Submit ~value:"Click" ()$</p> >>
 
 let suffixform2 = register_new_service ["suffixform2"] unit
@@ -504,7 +528,7 @@ let suffix5 =
            (head (title (pcdata "")) [])
            (body
               [p [pcdata "This is a page with suffix ";
-                  strong [pcdata (Ocsigen_lib.string_of_url_path s)]]])))
+                  strong [pcdata (Ocsigen_lib.string_of_url_path ~encode:false s)]]])))
 
 let nosuffix =
   register_new_service
@@ -926,6 +950,7 @@ let mainpage = register_new_service ["tests"] unit
          a sraise sp [pcdata "Wrong use of exceptions during service"] (); br ();
          a getcoex sp [pcdata "GET coservice with preapplied fallback, etc"] (); br ();
          a postcoex sp [pcdata "POST service with coservice fallback"] (); br ();
+         a su sp [pcdata "Suffix and other service at same URL"] (); br ();
          a preappliedsuffix sp [pcdata "Preapplied suffix"] (); br ();
          a getact sp [pcdata "action on GET attached coservice, etc"] 127; br ();
          a cookies sp [pcdata "Many cookies"] "le suffixe de l'URL"; br ();
