@@ -129,7 +129,7 @@ let split_regexp r s =
 
 
 (** permet de recuperer le fichier correspondant a l url *)
-let find_cgi_page reg sub_path =
+let find_cgi_page request reg sub_path =
   let find_file (filename, re, doc_root) =
     (* See also module Files in eliom.ml *)
     Ocsigen_messages.debug (fun () -> "--Cgimod: Testing \""^filename^"\".");
@@ -139,7 +139,7 @@ let find_cgi_page reg sub_path =
         if (stat.Unix.LargeFile.st_kind = Unix.S_DIR)
         then
           (Ocsigen_messages.debug2 "--Cgimod: this is a directory.";
-          raise Ocsigen_Is_a_directory)
+          raise (Ocsigen_Is_a_directory request))
         else filename
       in
       Ocsigen_messages.debug (fun () -> "--Cgimod: Looking for \""^filename^"\".");
@@ -159,7 +159,7 @@ let find_cgi_page reg sub_path =
     | Unix.Unix_error (Unix.ENOENT, _, _) -> raise Failed_404
   in
 
-  let sub_path = (Ocsigen_lib.string_of_url_path sub_path) in
+  let sub_path = (Ocsigen_lib.string_of_url_path ~encode:true sub_path) in
 
   match split_regexp reg.regexp sub_path with
   | None -> raise Failed_404
@@ -174,7 +174,7 @@ let find_cgi_page reg sub_path =
          path_info= string_conform0 path_info}
       in
       find_file (dr^sc, reg, dr)
-    with Not_found -> raise Failed_404 (* user not found *)
+    with Ocsigen_extensions.NoSuchUser -> raise Failed_404
 
 
 
@@ -454,39 +454,30 @@ let get_content str =
 (*****************************************************************************)
 let rec parse_global_config = function
   | [] -> ()
-  | (Element ("cgitimeout", [("value", s)], []))::ll ->
+  | (Element ("cgitimeout", [("value", s)], []))::[] ->
       cgitimeout := int_of_string s
   | _ -> raise (Error_in_config_file
                   ("Unexpected content inside cgimod config"))
 
-let _ = parse_global_config (Ocsigen_extensions.get_config ())
-
-
-
-
-(*****************************************************************************)
-(** A function that will create an error message from the exceptions
-    that may be raised during the initialisation phase, and raise again
-    all other exceptions. That function has type exn -> string. Use the
-   raise function if you don't need any. *)
-let exn_handler = raise
 
 (*****************************************************************************)
 
-let gen reg (charset, hostname, _, _) = function
-  | Ocsigen_extensions.Req_found (_, r) -> 
-      Lwt.return (Ocsigen_extensions.Ext_found r)
+let gen reg = function
+  | Ocsigen_extensions.Req_found _ -> 
+      Lwt.return Ocsigen_extensions.Ext_do_nothing
   | Ocsigen_extensions.Req_not_found (err, ri) ->
   catch
     (* Is it a cgi page? *)
     (fun () ->
          Ocsigen_messages.debug2 "--Cgimod: Is it a cgi file?";
          let (filename, re, doc_root) =
-           find_cgi_page reg ri.ri_sub_path
+           find_cgi_page ri reg ri.request_info.ri_sub_path
          in
          recupere_cgi
-           (ri.ri_method = Http_header.HEAD)
-           re doc_root filename ri hostname >>= fun (frame, finalizer) ->
+           (ri.request_info.ri_method = Http_header.HEAD)
+           re doc_root filename ri.request_info
+           (Ocsigen_extensions.get_hostname ri)
+         >>= fun (frame, finalizer) ->
          let header = frame.Ocsigen_http_frame.header in
          let content = get_content frame in
          Ocsigen_stream.add_finalizer content finalizer;
@@ -517,7 +508,8 @@ let gen reg (charset, hostname, _, _) = function
                   Ocsigen_stream.finalize content >>= fun () ->
                   if loc <> "" && loc.[0] = '/' then
                     Lwt.return
-                      (Ext_retry_with (ri_of_url loc ri,
+                      (Ext_retry_with ({ ri with request_info =
+                                           ri_of_url loc ri.request_info },
                                        Ocsigen_http_frame.Cookies.empty))
                   else
                     let default_result = Ocsigen_http_frame.default_result () in
@@ -544,7 +536,7 @@ let gen reg (charset, hostname, _, _) = function
                           Lwt.return
                             {default_result with
                                res_content_length = None;
-                               res_stream = content;
+                               res_stream = (content, None);
                                res_stop_stream = frame.Ocsigen_http_frame.abort;
 (*VVV NO! If sending is interrupted, we probably must do something else! *)
                                res_location= loc;
@@ -556,9 +548,7 @@ let gen reg (charset, hostname, _, _) = function
            (fun e -> Ocsigen_stream.finalize content >>= fun () -> Lwt.fail e))
     (function
       | Unix.Unix_error (Unix.EACCES,_,_)
-      | Ocsigen_malformed_url
       | Lost_connection _ as e -> fail e
-      | Ocsigen_Is_a_directory
       | Unix.Unix_error (Unix.ENOENT,_,_) -> return (Ext_next 404)
       | Failed_403 -> return (Ext_next 403)
       | Failed_404 -> return (Ext_next 404)
@@ -580,7 +570,7 @@ let rec set_env = function
      else (vr,vl)::set_env l
   | _ :: l -> raise (Error_in_config_file "Bad config tag for <cgi>")
 
-let parse_config path (a, hostname, b, c) _ parse_site = function
+let parse_config path _ _ = function
   | Element ("cgi", atts, l) ->
       let good_root r =
         Regexp.quote (string_conform2 r) 
@@ -596,7 +586,8 @@ let parse_config path (a, hostname, b, c) _ parse_site = function
            doc_root= Ocsigen_extensions.parse_user_dir (string_conform1 s);
            script= Ocsigen_extensions.parse_user_dir "$1";
 
-           path= string_conform (Ocsigen_lib.string_of_url_path path);
+           path= string_conform 
+          (Ocsigen_lib.string_of_url_path ~encode:true path);
            path_info="";
 
            exec=None;
@@ -608,7 +599,8 @@ let parse_config path (a, hostname, b, c) _ parse_site = function
            doc_root= Ocsigen_extensions.parse_user_dir (string_conform1 d);
            script= Ocsigen_extensions.parse_user_dir t;
 
-           path= string_conform (Ocsigen_lib.string_of_url_path path);
+           path= string_conform 
+              (Ocsigen_lib.string_of_url_path ~encode:true path);
            path_info=""; (* unknown for the moment *)
 
            exec= (match q with
@@ -619,28 +611,18 @@ let parse_config path (a, hostname, b, c) _ parse_site = function
            env=set_env l}
       | _ -> raise (Error_in_config_file "Wrong attributes for <cgi>")
       in
-      gen dir (a, hostname, b, c)
+      gen dir
   | Element (t, _, _) -> raise (Bad_config_tag_for_extension t)
   | _ ->
       raise (Error_in_config_file "Unexpected data in config file")
 
 
 
-(*****************************************************************************)
-(** Function to be called at the beginning of the initialisation phase
-    of the server  *)
-let start_init () =
-  ()
-
-(** Function to be called at the end of the initialisation phase *)
-let end_init () = ()
-(*      for default cgi dir *)
 
 (*****************************************************************************)
 (** Registration of the extension *)
-let _ = register_extension
-  (fun hostpattern -> parse_config)
-  void_extension
-  start_init
-  end_init
-  exn_handler
+let () = register_extension
+  ~name:"cgimod"
+  ~fun_site:(fun _ -> parse_config)
+  ~init_fun:parse_global_config
+  ()
