@@ -35,26 +35,18 @@ exception Ocsipersist_error
 
 
 (*****************************************************************************)
-(** Internal functions: storage directory *)
 
 open Simplexmlparser
 (** getting the directory from config file *)
-let rec parse_global_config d = function
-  | [] -> d
-  | (Element ("database", [("file", s)], []))::ll ->
-      (match d with
-      | None  -> parse_global_config (Some s) ll
-      | (Some _) -> raise (Ocsigen_extensions.Error_in_config_file
-                             ("Ocsipersist: Duplicate <store> tag")))
-  | (Element (tag,_,_))::ll -> parse_global_config d ll
+let rec parse_global_config = function
+  | [] -> None
+  | (Element ("database", [("file", s)], []))::[] -> Some s
   | _ -> raise (Ocsigen_extensions.Error_in_config_file
                   ("Unexpected content inside Ocsipersist config"))
 
-let db_file =
-  match parse_global_config None (Ocsigen_extensions.get_config ()) with
-  | None -> (Ocsigen_config.get_datadir ())^"/ocsidb"
-  | Some d -> d
-
+(* This reference is overwritten when the init function (at the end of the file)
+   is run, which occurs when the extension is loaded *)
+let db_file = ref ((Ocsigen_config.get_datadir ())^"/ocsidb")
 
 
 (*****************************************************************************)
@@ -81,7 +73,7 @@ let exec_safely f =
   let aux () =
    let db =
      Mutex.lock m ;
-     try db_open db_file with e -> Mutex.unlock m; raise e
+     try db_open !db_file with e -> Mutex.unlock m; raise e
    in
     (try
       let r = f db in
@@ -322,7 +314,26 @@ let length table =
   db_length table
 
 
+(* Registration of the extension *)
+
+let init config =
+  db_file := Ocsigen_config.get_datadir () ^"/ocsidb";
+  (match parse_global_config config with
+    | None -> ()
+    | Some d -> db_file := d
+  );
+  (* We check that we can access the database *)
+  try Lwt_unix.run (exec_safely (fun _ -> ()))
+  with e ->
+    Ocsigen_messages.errlog
+      (Printf.sprintf
+         "Error opening database file '%s' when registering Ocsipersist. \
+          Check that the directory exists, and that Ocsigen has enough \
+          rights" !db_file);
+    raise e
 
 
-
-
+let _ = Ocsigen_extensions.register_extension
+  ~name:"ocsipersist"
+  ~init_fun:init
+  ()
