@@ -34,98 +34,87 @@
    - enhance pipelining
    - HTTP/1.0
    - ...
-*)
 
 
-(* To compile it:
-ocamlfind ocamlc  -thread -package netstring,ocsigen -c revproxy.ml
-
-Then load it dynamically from Ocsigen's config file:
-   <extension module=".../revproxy.cmo"/>
-
+   - Make possible to return for example (Ext_next 404) to allow
+   other extensions to take the request?
+   There is a problem if the body contains data (POST request) ...
+   this data has been sent and is lost ...
 *)
 
 open Lwt
 open Ocsigen_extensions
 open Simplexmlparser
 
-exception Not_concerned
-
+exception Bad_answer_from_http_server
 
 
 (*****************************************************************************)
 (* The table of redirections for each virtual server                         *)
 type redir =
     { regexp: Netstring_pcre.regexp;
-      https: bool;
-      server:string;
-      port: string;
-      uri: string;
-      pipeline: bool}
-
-
-
-(*****************************************************************************)
-let rec parse_global_config = function
-  | [] -> ()
-  | _ -> raise (Error_in_config_file
-                  ("Unexpected content inside revproxy config"))
-
-let _ = parse_global_config (Ocsigen_extensions.get_config ())
-
-
+      full_url: Ocsigen_lib.yesnomaybe;
+      dest: string;
+      pipeline: bool;
+      keephost: bool}
 
 
 (*****************************************************************************)
 (* Finding redirections *)
 
-let find_redirection r path =
-  match Netstring_pcre.string_match r.regexp path 0 with
-  | None -> raise Not_concerned
-  | Some _ -> (* Matching regexp found! *)
-      (r.https,
-       Netstring_pcre.replace_first r.regexp r.server path,
-       int_of_string
-         (Netstring_pcre.global_replace r.regexp r.port path),
-       Netstring_pcre.global_replace r.regexp r.uri path)
 
-
-
-
-(*****************************************************************************)
-(** Function to be called at the beginning of the initialisation phase
-    of the server (actually each time the config file is reloaded) *)
-let start_init () =
-  ()
-
-(** Function to be called at the end of the initialisation phase *)
-let end_init () =
-  ()
-
-
-
-(*****************************************************************************)
 (** The function that will generate the pages from the request. *)
-exception Bad_answer_from_http_server
 
-let gen dir charset = function
-| Ocsigen_extensions.Req_found (_, r) -> Lwt.return (Ocsigen_extensions.Ext_found r)
+let gen dir = function
+| Ocsigen_extensions.Req_found _ -> 
+    Lwt.return Ocsigen_extensions.Ext_do_nothing
 | Ocsigen_extensions.Req_not_found (err, ri) ->
   catch
     (* Is it a redirection? *)
     (fun () ->
        Ocsigen_messages.debug2 "--Revproxy: Is it a redirection?";
-       let (https, host, port, uri) =
-         find_redirection dir
-           (match ri.ri_get_params_string with
-           | None -> ri.ri_sub_path_string
-           | Some g -> ri.ri_sub_path_string ^ "?" ^ g)
+       let dest =
+         let ri = ri.request_info in
+         let fi full =
+           Ocsigen_extensions.find_redirection
+             dir.regexp
+             full
+             dir.dest
+             ri.ri_ssl
+             ri.ri_host
+             ri.ri_server_port
+             ri.ri_get_params_string
+             ri.ri_sub_path_string
+             ri.ri_full_path_string
+         in 
+         match dir.full_url with
+           | Ocsigen_lib.Yes -> fi true
+           | Ocsigen_lib.No -> fi false
+           | Ocsigen_lib.Maybe -> 
+               try fi false 
+               with Ocsigen_extensions.Not_concerned -> fi true
        in
-       let uri = "/"^uri in
-       Ocsigen_messages.debug (fun () ->
-                         "--Revproxy: YES! Redirection to "^
-                           (if https then "https://" else "http://")^host^":"^
-                                (string_of_int port)^uri);
+       let (https, host, port, uri) = 
+         try
+           match Ocsigen_lib.parse_url dest with
+             | (Some https, Some host, port, uri, _, _, _, _) -> 
+                 let port = match port with
+                   | None -> if https then 443 else 80
+                   | Some p -> p
+                 in
+                 (https, host, port, uri)
+             | _ -> raise (Ocsigen_extensions.Error_in_config_file
+                             ("Revproxy : error in destination URL "^dest))
+(*VVV catch only Neturl exceptions! *)
+         with e -> raise (Ocsigen_extensions.Error_in_config_file
+                            ("Revproxy : error in destination URL "^dest^" - "^
+                               Printexc.to_string e))
+       in
+       Ocsigen_messages.debug
+         (fun () ->
+            "--Revproxy: YES! Redirection to "^
+              (if https then "https://" else "http://")^host^":"^
+              (string_of_int port)^uri);
 
        Ocsigen_lib.get_inet_addr host >>= fun inet_addr ->
 
@@ -134,7 +123,23 @@ let gen dir charset = function
           => We return.
        *)
 
+       let host = 
+         match
+           if dir.keephost 
+           then match ri.request_info.ri_host with 
+             | Some h -> Some h
+             | None -> None
+           else None 
+         with
+           | Some h -> h
+           | None -> 
+               if (not https && port=80) || (https && port=443)
+               then host
+               else host^":"^string_of_int port
+       in
+
        let do_request =
+         let ri = ri.request_info in
          if dir.pipeline then
            Ocsigen_http_client.raw_request
              ~headers:ri.ri_http_frame.Ocsigen_http_frame.header.Ocsigen_http_frame.Http_header.headers
@@ -185,11 +190,10 @@ let gen dir charset = function
                      in
                      Lwt.return
                        {empty_result with
-                        Ocsigen_http_frame.res_content_length = length;
-                        Ocsigen_http_frame.res_headers= headers;
-                        Ocsigen_http_frame.res_stop_stream =
-                        http_frame.Ocsigen_http_frame.abort;
-                        Ocsigen_http_frame.res_code= code;
+                          Ocsigen_http_frame.res_content_length = length;
+                          res_headers= headers;
+                          res_stop_stream = http_frame.Ocsigen_http_frame.abort;
+                          res_code= code;
                        }
                  | Some stream ->
                      let default_result =
@@ -200,12 +204,12 @@ let gen dir charset = function
                      in
                      Lwt.return
                        {default_result with
-                        Ocsigen_http_frame.res_content_length = length;
-                        Ocsigen_http_frame.res_stream = stream;
-                        Ocsigen_http_frame.res_stop_stream =
-                        http_frame.Ocsigen_http_frame.abort;
-                        Ocsigen_http_frame.res_headers= headers;
-                        Ocsigen_http_frame.res_code= code;
+                          Ocsigen_http_frame.res_content_length = length;
+                          res_stream = (stream, None);
+                          res_stop_stream =
+                           http_frame.Ocsigen_http_frame.abort;
+                          res_headers= headers;
+                          res_code= code;
                        }
             )
          )
@@ -218,81 +222,58 @@ let gen dir charset = function
 
 
 (*****************************************************************************)
-(** Configuration for each site.
-    These tags are inside <site ...>...</site> in the config file.
 
-   For example:
-   <site dir="">
-     <revproxy regexp="" ... />
-   </extension>
 
- *)
-
-let parse_config path charset _ parse_site = function
+let parse_config = function
   | Element ("revproxy", atts, []) ->
-      let rec parse_attrs ((r, s, prot, port, u, pipeline) as res) = function
+      let rec parse_attrs ((r, f, d, pipeline, h) as res) = function
         | [] -> res
-        | ("regexp", regexp)::l when r = None ->
+        | ("regexp", regexp)::l when r = None -> (* deprecated *)
             parse_attrs
-              (Some (Netstring_pcre.regexp ("^"^regexp^"$")), s, prot, port, u, pipeline)
+              (Some (Netstring_pcre.regexp ("^"^regexp^"$")), Ocsigen_lib.Maybe,
+               d, pipeline, h)
               l
-        | ("protocol", protocol)::l
-          when prot = None && String.lowercase protocol = "http" ->
+        | ("fullurl", regexp)::l when r = None ->
             parse_attrs
-              (r, s, Some false, port, u, pipeline)
+              (Some (Netstring_pcre.regexp ("^"^regexp^"$")), Ocsigen_lib.Yes,
+               d, pipeline, h)
               l
-        | ("protocol", protocol)::l
-          when prot = None && String.lowercase protocol = "https" ->
+        | ("suburl", regexp)::l when r = None ->
             parse_attrs
-              (r, s, Some true, port, u, pipeline)
+              (Some (Netstring_pcre.regexp ("^"^regexp^"$")), Ocsigen_lib.No,
+               d, pipeline, h)
               l
-        | ("server", server)::l when s = None ->
+        | ("dest", dest)::l when d = None ->
             parse_attrs
-              (r, Some server, prot, port, u, pipeline)
+              (r, f, Some dest, pipeline, h)
               l
-        | ("uri", uri)::l when u = None ->
+        | ("keephost", "keephost")::l ->
             parse_attrs
-              (r, s, prot, port, Some uri, pipeline)
-              l
-        | ("port", p)::l when port = None ->
-            parse_attrs
-              (r, s, prot, Some p, u, pipeline)
+              (r, f, d, pipeline, true)
               l
         | ("nopipeline", "nopipeline")::l ->
             parse_attrs
-              (r, s, prot, port, u, false)
+              (r, f, d, false, h)
               l
-        | _ -> raise (Error_in_config_file "Wrong attribute for <revproxy>")
-        in
-        let dir =
-          match parse_attrs (None, None, None, None, None, true) atts with
-          | (None, _, _, _, _, _) -> raise (Error_in_config_file "Missing attribute regexp for <revproxy>")
-          | (_, None, _, _, _, _) -> raise (Error_in_config_file "Missing attribute server for <revproxy>")
-          | (_, _, _, _, None, _) -> raise (Error_in_config_file "Missing attribute uri for <revproxy>")
-          | (Some r, Some s, None, port, Some u, pipeline) ->
+        | (a, _) :: _ ->
+            badconfig "Wrong or duplicate attribute '%s' for <revproxy>" a
+      in
+      let dir =
+          match parse_attrs (None, Ocsigen_lib.Yes, None, true, false) atts with
+          | (None, _, _, _, _) ->
+              badconfig "Missing attribute 'regexp' for <revproxy>"
+          | (_, _, None, _, _) ->
+              badconfig "Missing attribute 'dest' for <revproxy>"
+          | (Some r, full, Some d, pipeline, h) ->
               {
-               regexp=r;
-               server=s;
-               https=false;
-               port=(match port with
-               | Some p -> p
-               | None -> "80");
-               uri=u;
-               pipeline=pipeline;
-             }
-          | (Some r, Some s, Some prot, port, Some u, pipeline) ->
-              {
-               regexp=r;
-               server=s;
-               https=prot;
-               port=(match port with
-               | Some p -> p
-               | None -> if prot then "443" else "80");
-               uri=u;
-               pipeline=pipeline;
+                regexp=r;
+                full_url=full;
+                dest=d;
+                pipeline=pipeline;
+                keephost=h;
              }
         in
-        gen dir charset
+        gen dir
   | Element (t, _, _) -> raise (Bad_config_tag_for_extension t)
   | _ -> raise (Error_in_config_file "(revproxy extension) Bad data")
 
@@ -302,12 +283,10 @@ let parse_config path charset _ parse_site = function
 
 (*****************************************************************************)
 (** Registration of the extension *)
-let _ = register_extension
-    ~respect_pipeline:true (* We ask ocsigen to respect pipeline order
-                              when sending to extensions! *)
-    (fun hostpattern -> parse_config)
-    (fun hostpattern -> parse_config)
-    start_init
-    end_init
-    raise
-
+let () = register_extension
+  ~name:"revproxy"
+  ~fun_site:(fun _ _ _ _ -> parse_config)
+  ~user_fun_site:(fun _ _ _ _ _ -> parse_config)
+  ~respect_pipeline:true (* We ask ocsigen to respect pipeline order
+                            when sending to extensions! *)
+  ()
