@@ -34,6 +34,8 @@ module Old_Xhtml_content =
   struct
     type t = [ `Html ] XHTML.M.elt
 
+    type options = unit
+
     let get_etag_aux x =
       Some (Digest.to_hex (Digest.string x))
 
@@ -41,7 +43,7 @@ module Old_Xhtml_content =
       let x = Xhtmlpretty.xhtml_print c in
       get_etag_aux x
 
-    let result_of_content c =
+    let result_of_content ?(options = ()) c =
       let x = Xhtmlpretty.xhtml_print c in
       let md5 = get_etag_aux x in
       let default_result = default_result () in
@@ -52,9 +54,10 @@ module Old_Xhtml_content =
          res_etag = md5;
          res_headers= Http_headers.dyn_headers;
          res_stream =
-         Ocsigen_stream.make
-           (fun () -> Ocsigen_stream.cont x
-               (fun () -> Ocsigen_stream.empty None))
+            (Ocsigen_stream.make
+               (fun () -> Ocsigen_stream.cont x
+                  (fun () -> Ocsigen_stream.empty None)),
+             None)
        }
 
   end
@@ -62,7 +65,7 @@ module Old_Xhtml_content =
 module Xhtml_content_(Xhtmlprinter : sig
                         val xhtml_stream :
                           ?version:[< `HTML_v03_02 | `HTML_v04_01
-                          | `XHTML_01_00 | `XHTML_01_01
+                          | `XHTML_01_00 | `XHTML_01_01 | `Doctype of string
                               > `XHTML_01_01 ] ->
                           ?width:int ->
                           ?encode:(string -> string) ->
@@ -72,12 +75,14 @@ module Xhtml_content_(Xhtmlprinter : sig
   struct
     type t = [ `Html ] XHTML.M.elt
 
+    type options = [ `HTML_v03_02 | `HTML_v04_01 | `XHTML_01_00 | `XHTML_01_01 | `Doctype of string ]
+
     let get_etag_aux x = None
 
     let get_etag c = None
 
-    let result_of_content c =
-      let x = Xhtmlprinter.xhtml_stream c in
+    let result_of_content ?(options = `XHTML_01_01) c =
+      let x = Xhtmlprinter.xhtml_stream ~version:options c in
       let default_result = default_result () in
       Lwt.return
         {default_result with
@@ -85,23 +90,25 @@ module Xhtml_content_(Xhtmlprinter : sig
          res_content_type = Some "text/html";
          res_etag = get_etag c;
          res_headers= Http_headers.dyn_headers;
-         res_stream = x
+         res_stream = (x, None)
        }
 
   end
 
-module Xhtml_content = Xhtml_content_(Xhtmlpretty)
-module Xhtmlcompact_content = Xhtml_content_(Xhtmlcompact)
+module Xhtml_content = Xhtml_content_(Xhtmlpretty_streams)
+module Xhtmlcompact_content = Xhtml_content_(Xhtmlcompact_streams)
 
 (*****************************************************************************)
 module Text_content =
   struct
     type t = string (* content *) * string (* content-type *)
 
+    type options = unit
+
     let get_etag (x, _) =
       Some (Digest.to_hex (Digest.string x))
 
-    let result_of_content ((c, ct) as content) =
+    let result_of_content ?(options = ()) ((c, ct) as content) =
       let md5 = get_etag content in
       let default_result = default_result () in
       Lwt.return
@@ -111,10 +118,12 @@ module Text_content =
          res_content_type = Some ct;
          res_headers= Http_headers.dyn_headers;
          res_stream =
-         Ocsigen_stream.make
-           (fun () -> Ocsigen_stream.cont c (fun () -> Ocsigen_stream.empty None))
-
-       }
+            (Ocsigen_stream.make
+               (fun () -> 
+                  Ocsigen_stream.cont c (fun () -> Ocsigen_stream.empty None)),
+             None)
+               
+            }
 
   end
 
@@ -124,15 +133,17 @@ module Stream_content =
   struct
     type t = string Ocsigen_stream.t
 
+    type options = unit
+
     let get_etag c = None
 
-    let result_of_content c =
+    let result_of_content ?(options = ()) c =
       let default_result = default_result () in
       Lwt.return
         {default_result with
          res_content_length = None;
          res_headers= Http_headers.dyn_headers;
-         res_stream = c}
+         res_stream = (c, None)}
 
   end
 
@@ -143,9 +154,11 @@ module Streamlist_content =
     type t = (unit -> string Ocsigen_stream.t Lwt.t) list
           * string (* content-type *)
 
+    type options = unit
+
     let get_etag c = None
 
-    let result_of_content (c, ct) =
+    let result_of_content ?(options = ()) (c, ct) =
       let finalizer = ref (fun () -> Lwt.return ()) in
       let finalize () =
         let f = !finalizer in
@@ -186,7 +199,8 @@ module Streamlist_content =
         {default_result with
          res_content_length = None;
          res_etag = get_etag c;
-         res_stream = Ocsigen_stream.make ~finalize (fun () -> next_stream c);
+         res_stream = 
+            (Ocsigen_stream.make ~finalize (fun () -> next_stream c), None);
          res_headers= Http_headers.dyn_headers;
          res_content_type = Some ct}
 
@@ -198,73 +212,26 @@ module Empty_content =
   struct
     type t = unit
 
+    type options = unit
+
     let get_etag c = None
 
-    let result_of_content c = Lwt.return (empty_result ())
+    let result_of_content ?(options = ()) c = Lwt.return (empty_result ())
 
   end
 
 (*****************************************************************************)
 (* Files *)
-let mimeht = Hashtbl.create 600
-
-let parse_mime_types filename =
-  let rec read_and_split in_ch =
-    try
-      let line = input_line in_ch in
-      let line_upto =
-        try
-          let upto = String.index line '#' in
-          String.sub line 0 upto
-        with Not_found -> line
-      in
-      let strlist =
-        Netstring_pcre.split (Netstring_pcre.regexp "\\s+") line_upto
-      in
-      match  List.length strlist with
-      | 0 | 1 -> read_and_split in_ch
-      | _ ->
-          let make_pair = (fun h -> Hashtbl.add mimeht h (List.hd strlist)) in
-          List.iter make_pair (List.tl strlist);
-          read_and_split in_ch
-    with End_of_file -> ()
-  in
-  try
-    let in_ch = open_in filename in
-    (try
-      read_and_split in_ch
-    with e -> close_in in_ch; raise e);
-    close_in in_ch
-  with Sys_error _ | Not_found -> ()
-
-
-let rec affiche_mime () =
-  Hashtbl.iter (fun f s -> Ocsigen_messages.debug (fun () -> f^" "^s)) mimeht
-
-
-(* send a file in an HTTP frame*)
-let content_type_from_file_name =
-  let parsed = ref false in
-  fun filename ->
-    if not !parsed
-    then begin
-      parsed := true;
-      parse_mime_types (Ocsigen_config.get_mimefile ());
-    end;
-    try
-      let pos = (String.rindex filename '.') in
-      let extens =
-        String.sub filename
-          (pos+1)
-          ((String.length filename) - pos - 1)
-      in Hashtbl.find mimeht extens
-    with Not_found | Invalid_argument _ -> "application/octet-stream"
-(*VVV Make the defaultcontent-type configurable! *)
 
 (** this module instanciate the HTTP_CONTENT signature for files *)
 module File_content =
   struct
-    type t = string (* nom du fichier *)
+    type t =
+        string (* nom du fichier *) *
+        Ocsigen_charset_mime.charset_assoc *
+        Ocsigen_charset_mime.mime_assoc
+
+    type options = unit
 
     let read_file ?buffer_size fd =
       let buffer_size = match buffer_size with
@@ -274,13 +241,13 @@ module File_content =
       Ocsigen_messages.debug2 "start reading file (file opened)";
       let buf = String.create buffer_size in
       let rec read_aux () =
-          Lwt_unix.read fd buf 0 buffer_size >>= fun lu ->
-          if lu = 0 then
+          Lwt_unix.read fd buf 0 buffer_size >>= fun read ->
+          if read = 0 then
             Ocsigen_stream.empty None
           else begin
-            if lu = buffer_size
+            if read = buffer_size
             then Ocsigen_stream.cont buf read_aux
-            else Ocsigen_stream.cont (String.sub buf 0 lu) read_aux
+            else Ocsigen_stream.cont (String.sub buf 0 read) read_aux
           end
       in read_aux
 
@@ -288,37 +255,48 @@ module File_content =
       Some (Printf.sprintf "%Lx-%x-%f" st.Unix.LargeFile.st_size
               st.Unix.LargeFile.st_ino st.Unix.LargeFile.st_mtime)
 
-    let get_etag f =
+    let get_etag (f, _, _) =
       let st = Unix.LargeFile.stat f in
       get_etag_aux st
 
-    let result_of_content c =
+    let skip fd stream k =
+      try
+        ignore 
+          (Unix.LargeFile.lseek (Lwt_unix.unix_file_descr fd) k Unix.SEEK_CUR);
+        Ocsigen_stream.next (Ocsigen_stream.get stream)
+      with e -> Lwt.fail e
+
+    let result_of_content ?options (c, charset_assoc, mime_assoc) =
       (* open the file *)
       try
-        let fd =
-          Lwt_unix.of_unix_file_descr
-            (Unix.openfile c [Unix.O_RDONLY;Unix.O_NONBLOCK] 0o666)
-        in
-        let st = Unix.LargeFile.stat c in
-        let etag = get_etag_aux st in
-        let stream = read_file fd in
-        let default_result = default_result () in
-        Lwt.return
-          {default_result with
-           res_content_length = Some st.Unix.LargeFile.st_size;
-           res_content_type = Some (content_type_from_file_name c);
-           res_lastmodified = Some st.Unix.LargeFile.st_mtime;
-           res_etag = etag;
-           res_stream =
-           Ocsigen_stream.make
-             ~finalize:
-             (fun () ->
-               Ocsigen_messages.debug2 "closing file";
-               Lwt_unix.close fd;
-               return ())
-             stream
-         }
-      with e -> fail e
+        let fdu = Unix.openfile c [Unix.O_RDONLY;Unix.O_NONBLOCK] 0o666 in
+        let fd = Lwt_unix.of_unix_file_descr fdu in
+        try
+          let st = Unix.LargeFile.fstat fdu in
+          let etag = get_etag_aux st in
+          let stream = read_file fd in
+          let default_result = default_result () in
+          Lwt.return
+            {default_result with
+               res_content_length = Some st.Unix.LargeFile.st_size;
+               res_content_type =
+                Some (Ocsigen_charset_mime.find_mime c mime_assoc);
+               res_charset =
+                Some (Ocsigen_charset_mime.find_charset c charset_assoc);
+               res_lastmodified = Some st.Unix.LargeFile.st_mtime;
+               res_etag = etag;
+               res_stream =
+                (Ocsigen_stream.make
+                   ~finalize:
+                   (fun () ->
+                      Ocsigen_messages.debug2 "closing file";
+                      Lwt_unix.close fd;
+                      return ())
+                   stream,
+                 Some (skip fd))
+            }
+        with e -> Lwt_unix.close fd; raise e
+      with e -> Ocsigen_messages.debug2 (Printexc.to_string e);  fail e
 
   end
 
@@ -330,6 +308,8 @@ module Directory_content =
   struct
     type t = string (* dir name *) * string list (* corresponding URL path *)
 
+    type options = unit
+
     let get_etag_aux st =
       Some (Printf.sprintf "%Lx-%x-%f" st.Unix.LargeFile.st_size
               st.Unix.LargeFile.st_ino st.Unix.LargeFile.st_mtime)
@@ -382,43 +362,36 @@ module Directory_content =
         | _ -> "/ocsigenstuff/unknown.png"
 
 
+    (* An html row for a file in the directory listing *)
+    let file_row name icon stat = Printf.sprintf "
+<tr>
+  <td class=\"img\"><img src=\"%s\" alt=\"\" /></td>
+  <td><a href=\"%s\">%s</a></td>
+  <td>%Ld</td>
+  <td>%s</td>
+</tr>"
+      icon (Netencoding.Url.encode ~plus:false name) name
+      stat.Unix.LargeFile.st_size (date stat.Unix.LargeFile.st_mtime)
+
 
     let directory filename =
       let dir = Unix.opendir filename in
       let rec aux d =
         try
           let f = Unix.readdir dir in
-          let stat = Unix.LargeFile.stat (filename^f) in
-          if (stat.Unix.LargeFile.st_kind = Unix.S_DIR && f <> "." && f <> "..")
-          then
-            (
-             `Dir, f, (
-             "<tr>\n"^
-             "<td class=\"img\"><img src=\"/ocsigenstuff/folder_open.png\" alt=\"\" /></td>\n"^
-             "<td><a href=\""^f^"\">"^f^"</a></td>\n"^
-             "<td>"^(Int64.to_string stat.Unix.LargeFile.st_size)^"</td>\n"^
-             "<td>"^(date stat.Unix.LargeFile.st_mtime)^"</td>\n"^
-             "</tr>\n")
-            )::aux d
-          else
-            if (stat.Unix.LargeFile.st_kind
-                  = Unix.S_REG)
+          try
+            let stat = Unix.LargeFile.stat (filename^f) in
+            if stat.Unix.LargeFile.st_kind = Unix.S_DIR && f <> "." && f <> ".."
             then
-              (
-               if f.[(String.length f) - 1] = '~'
-               then aux d
-               else
-                 (
-                  `Reg, f,
-                  "<tr>\n"^
-                  "<td class=\"img\"><img src=\""^image_found f^"\" alt=\"\" /></td>\n"^
-                  "<td><a href=\""^f^"\">"^f^"</a></td>\n"^
-                  "<td>"^(Int64.to_string stat.Unix.LargeFile.st_size)^"</td>\n"^
-                  "<td>"^(date stat.Unix.LargeFile.st_mtime)^"</td>\n"^
-                  "</tr>\n"
-                 )::aux d
-              )
-            else aux d
+              (`Dir, f,
+               file_row f "/ocsigenstuff/folder_open.png" stat) :: aux d
+            else
+              if stat.Unix.LargeFile.st_kind = Unix.S_REG &&
+                f.[(String.length f) - 1] <> '~'
+              then
+                (`Reg, f, file_row f (image_found f) stat) :: aux d
+              else aux d
+          with _ (* Unix.stat can fail for a lot of reasons *) -> aux d
         with
           End_of_file -> Unix.closedir d;[]
 
@@ -443,31 +416,35 @@ module Directory_content =
 
 
 
-    let result_of_content (filename, path) =
+    let result_of_content ?(options = ()) (filename, path) =
       let stat = Unix.LargeFile.stat filename in
       let rec back = function
-        | [] -> assert false
-        | [a] -> "/"
-        | [a;""] -> "/"
-        | i::j -> "/"^i^(back j)
+        | [] | [""] -> assert false
+        | [_] | [_ ; ""] -> []
+        | i::j -> i :: (back j)
       in
       let parent =
-        if (path= []) || (path = [""])
-        then "/"
-        else back path
+        if path = [] || path = [""] then
+          None
+        else
+          Some ("/"^Ocsigen_lib.string_of_url_path ~encode:true (back path))
       in
       let before =
-        let st = (Ocsigen_lib.string_of_url_path path) in
+        let st = (Ocsigen_lib.string_of_url_path ~encode:true path) in
         "<html>\n\
-         <head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n\
+         <head><meta http-equiv=\"Content-Type\" content=\"text/html;\" />\n\
          <link rel=\"stylesheet\" type=\"text/css\" href=\"/ocsigenstuff/style.css\" media=\"screen\" />\n\
          <title>Listing Directory: "^st^"</title>\n</head>\n\
          <body><h1>"^st^"</h1>\n\
          <table summary=\"Contenu du dossier "^st^"\">\n\
          <tr id=\"headers\"><th></th><th>Name</th><th>Size</th>\
-         <th>Last modified</th></tr>\
-         <tr>\n\
-         <td class=\"img\"><img src=\"/ocsigenstuff/back.png\" alt=\"\" /></td>\n\
+         <th>Last modified</th></tr>\n"
+
+      and back = match parent with
+        | None -> ""
+        | Some parent ->
+         "<tr>\n\
+          <td class=\"img\"><img src=\"/ocsigenstuff/back.png\" alt=\"\" /></td>\n\
          <td><a href=\""^parent^"\">Parent Directory</a></td>\n\
          <td>"^(Int64.to_string stat.Unix.LargeFile.st_size)^"</td>\n\
          <td>"^(date stat.Unix.LargeFile.st_mtime)^"</td>\n\
@@ -478,7 +455,7 @@ module Directory_content =
          <p id=\"footer\">Ocsigen Webserver</p>\
          </body></html>"
       in
-      let c = before^(directory filename)^after in
+      let c = before^back^(directory filename)^after in
       let etag = get_etag_aux stat in
       Text_content.result_of_content (c, "text/html") >>= fun r ->
       Lwt.return
@@ -499,6 +476,8 @@ module Error_content =
   struct
     type t = int option * exn option * Ocsigen_http_frame.cookieset
 
+    type options = unit
+
     let get_etag c = None
 
     let error_page s msg c =
@@ -510,7 +489,7 @@ module Error_content =
             c)
         )
 
-    let result_of_content (code, exn, cookies_to_set) =
+    let result_of_content ?(options = ()) (code, exn, cookies_to_set) =
       let code = match code with
       | None -> 500
       | Some c -> c
