@@ -25,19 +25,31 @@
 
 (** Writing extensions for Ocsigen                                           *)
 
+(* TODO
+
+   - awake must be called after each Ext_found or Ext_found_continue_with
+   or Ext_found_stop sent by an extension. It is perhaps called too often.
+
+*)
 
 open Lwt
 open Ocsigen_lib
 
 exception Ocsigen_http_error of (Ocsigen_http_frame.cookieset * int)
-exception Ocsigen_Is_a_directory
-exception Ocsigen_malformed_url
 exception Ocsigen_Internal_Error of string
 exception Ocsigen_Looping_request
 
+
+(** Xml tag not recognized by an extension (usually not a real error) *)
 exception Bad_config_tag_for_extension of string
+
+(** Error in a <site> tag inside the main ocsigen.conf file *)
 exception Error_in_config_file of string
 
+(** Option incorrect in a userconf file *)
+exception Error_in_user_config_file of string
+
+
 let badconfig fmt = Printf.ksprintf (fun s -> raise (Error_in_config_file s)) fmt
 
 (*****************************************************************************)
@@ -63,7 +75,113 @@ let client_id = Ocsigen_http_com.connection_id
 let client_connection x = x
 let client_of_connection x = x
 
+
+(*****************************************************************************)
+
+(* Server configuration, for local files that must not be sent *)
+
+type do_not_serve = {
+  do_not_serve_regexps: string list;
+  do_not_serve_files: string list;
+  do_not_serve_extensions: string list;
+}
+
+(* BY TODO : Use unbalanced trees instead *)
+let join_do_not_serve d1 d2 = {
+  do_not_serve_regexps = d1.do_not_serve_regexps @ d2.do_not_serve_regexps;
+  do_not_serve_files = d1.do_not_serve_files @ d2.do_not_serve_files;
+  do_not_serve_extensions = d1.do_not_serve_extensions @ d2.do_not_serve_extensions;
+}
+
+let hash_consed_do_not_serve = Hashtbl.create 17
+
+exception IncorrectRegexpes of do_not_serve
+
+let do_not_serve_to_regexp d =
+  try Hashtbl.find hash_consed_do_not_serve d
+  with Not_found ->
+    let wrap l = if l = [] then None else Some l
+    and bind f = function None -> None | Some v -> Some (f v)
+    in
+    let files, extensions, regexps =
+      wrap d.do_not_serve_files,
+      wrap d.do_not_serve_extensions,
+      wrap d.do_not_serve_regexps
+    in
+    let paren_quote l =
+      String.concat "|" (List.map (fun s -> Printf.sprintf "(%s)"
+                                     (Netstring_pcre.quote s)) l)
+    and paren l =
+      String.concat "|" (List.map (fun s -> Printf.sprintf "(%s)"  s) l)
+    in
+    let files = bind paren_quote files
+    and extensions = bind paren_quote extensions
+    and regexps = bind paren regexps
+    in
+    let files = bind (Printf.sprintf ".*/(%s)") files
+    and extensions = bind (Printf.sprintf ".*\\.(%s)") extensions
+    in
+    let l = List.fold_left (fun r -> function None -> r | Some v -> v :: r)
+      [] [files;extensions;regexps]
+    in
+    let regexp =
+      if l = [] then
+        (* This regexp should not never match *) "$^"
+      else
+        Printf.sprintf "^(%s)$" (paren l)
+    in
+    (try
+       Ocsigen_messages.debug (fun () -> Printf.sprintf
+                                 "Compiling exclusion regexp %s" regexp);
+       let r = Netstring_pcre.regexp regexp in
+       Hashtbl.add hash_consed_do_not_serve d r;
+       r
+     with _ -> raise (IncorrectRegexpes d)
+    )
+
+
+(*****************************************************************************)
+
+(* Main server configuration *)
+
+
+type config_info = {
+  default_hostname: string;
+  default_httpport: int;
+  default_httpsport: int;
+
+  mime_assoc: Ocsigen_charset_mime.mime_assoc;
+
+  charset_assoc : Ocsigen_charset_mime.charset_assoc;
+
+  (** Default name to use as index file when a directory is requested.
+      Use [None] if no index should be tried. The various indexes
+      are tried in the given order. If no index is specified,
+      or the index does not exists, the content of the directory
+      might be listed, according to [list_directry_content] *)
+  default_directory_index : string list;
+
+  (** Should the list of files in a directory be displayed
+      if there is no index in this directory ? *)
+  list_directory_content : bool;
+
+  (** Should symlinks be followed when accessign a local file? *)
+  follow_symlinks: follow_symlink;
+
+  do_not_serve_404: do_not_serve;
+  do_not_serve_403: do_not_serve;
+}
+and follow_symlink =
+  | DoNotFollowSymlinks (** Never follow a symlink *)
+  | FollowSymlinksIfOwnerMatch (** Follow a symlink if the symlink and its
+                          target have the same owner *)
+  | AlwaysFollowSymlinks (** Always follow symlinks *)
+
+
+
 (* Requests *)
+type ifrange = IR_No | IR_Ifunmodsince of float | IR_ifmatch of string
+
 type request_info =
     {ri_url_string: string;
      ri_url: Neturl.url;
@@ -72,10 +190,13 @@ type request_info =
      ri_ssl: bool; (** true if HTTPS, false if HTTP *)
      ri_full_path_string: string; (** full path of the URL *)
      ri_full_path: string list;   (** full path of the URL *)
+     ri_original_full_path_string: string;   (** full path of the URL, as first sent by the client. Should not be changed by extensions, even rewritemod. It is used to create relative links. *)
+     ri_original_full_path: string list;   (** full path of the URL, as first sent by the client. See below. *)
      ri_sub_path: string list;   (** path of the URL (only part concerning the site) *)
      ri_sub_path_string: string;   (** path of the URL (only part concerning the site) *)
      ri_get_params_string: string option; (** string containing GET parameters *)
-     ri_host: string option; (** Host field of the request (if any) *)
+     ri_host: string option; (** Host field of the request (if any), without port *)
+     ri_port_from_host_field: int option; (** Port in the host field of the request (if any) *)
      ri_get_params: (string * string) list Lazy.t;  (** Association list of get parameters *)
      ri_initial_get_params: (string * string) list Lazy.t;  (** Association list of get parameters, as sent by the browser (must not be modified by extensions) *)
      ri_post_params: (string * string) list Lwt.t Lazy.t; (** Association list of post parameters *)
@@ -90,7 +211,7 @@ type request_info =
      ri_cookies: string Ocsigen_http_frame.Cookievalues.t Lazy.t;  (** Cookies sent by the browser *)
      ri_ifmodifiedsince: float option;   (** if-modified-since field *)
      ri_ifunmodifiedsince: float option;   (** if-unmodified-since field *)
-     ri_ifnonematch: string list;   (** if-none-match field ( * and weak entity tags not implemented) *)
+     ri_ifnonematch: string list option;   (** if-none-match field ( * and weak entity tags not implemented) *)
      ri_ifmatch: string list option;   (** if-match field ( * not implemented) *)
      ri_content_type: ((string * string) * (string * string) list) option; (** Content-Type HTTP header *)
      ri_content_type_string: string option; (** Content-Type HTTP header *)
@@ -108,10 +229,22 @@ type request_info =
                                       extensions
                                    *)
      ri_client: client; (** The request connection *)
+     ri_range: ((int64 * int64) list * int64 option * ifrange) option Lazy.t; 
+     (** Range HTTP header. [None] means all the document. 
+         List of intervals + possibly from an index to the end of the document.
+     *)
    }
+and request = {
+  request_info: request_info;
+  request_config: config_info;
+}
+
+exception Ocsigen_Is_a_directory of request
 
 
 type answer =
+  | Ext_do_nothing
+      (** Do nothing *)
   | Ext_found of (unit -> Ocsigen_http_frame.result Lwt.t)
       (** "OK stop! I will take the page.
           You can start the following request of the same pipelined connection.
@@ -120,8 +253,12 @@ type answer =
           when it is sure it is safe to start next request.
           Usually immediately. But in some case, for example proxies,
           you don't want the request of one connection to be handled in
-          different order.
+          different order. (for example revproxy.ml starts its requests
+          to another server before returning Ext_found, to ensure that all
+          requests are done in same order).
       *)
+  | Ext_found_stop of (unit -> Ocsigen_http_frame.result Lwt.t)
+      (** Found but do not try next extensions *)
   | Ext_next of int (** Page not found. Try next extension.
                         The integer is the HTTP error code.
                         It is usally 404, but may be for ex 403 (forbidden)
@@ -146,32 +283,45 @@ type answer =
                         do not try next host.
                         The integer is the HTTP error code, usally 403.
                      *)
-  | Ext_continue_with of (request_info * Ocsigen_http_frame.cookieset * int)
+  | Ext_continue_with of (request * Ocsigen_http_frame.cookieset * int)
         (** Used to modify the request before giving it to next extension.
-            The extension returns the request_info (possibly modified)
+            The extension returns the request (possibly modified)
             and a set of cookies if it wants to set or cookies
             ({!Ocsigen_http_frame.Cookies.empty} for no cookies).
-            You must add these cookies yourself in request_info if you
+            You must add these cookies yourself in request if you
             want them to be seen by subsequent extensions,
             for example using {!Ocsigen_http_frame.compute_new_ri_cookies}.
             The integer is usually equal to the error code received
             from preceding extension (but you may want to modify it).
          *)
-  | Ext_retry_with of request_info * Ocsigen_http_frame.cookieset
-        (** Used to retry all the extensions with a new request_info.
-            The extension returns the request_info (possibly modified)
+  | Ext_retry_with of request * Ocsigen_http_frame.cookieset
+        (** Used to retry all the extensions with a new request.
+            The extension returns the request (possibly modified)
             and a set of cookies if it wants to set or cookies
             ({!Ocsigen_http_frame.Cookies.empty} for no cookies).
-            You must add these cookies yourself in request_info if you
+            You must add these cookies yourself in request if you
             want them to be seen by subsequent extensions,
             for example using {!Ocsigen_http_frame.compute_new_ri_cookies}.
          *)
   | Ext_sub_result of extension2
-
+        (** Used if your extension want to define option that may contain
+            other options from other extensions.
+            In that case, while parsing the configuration file, call
+            the parsing function (of type [parse_fun]),
+            that will return something of type [extension2].
+        *)
+  | Ext_found_continue_with of 
+      (unit -> (Ocsigen_http_frame.result * request) Lwt.t)
+        (** Same as [Ext_found] but may modify the request. *)
+  | Ext_found_continue_with' of (Ocsigen_http_frame.result * request)
+        (** Same as [Ext_found_continue_with] but does not allow to delay
+            the computation of the page. You should probably not use it,
+            but for output filters.
+        *)
 
 and request_state =
-  | Req_not_found of (int * request_info)
-  | Req_found of (request_info * (unit -> Ocsigen_http_frame.result Lwt.t))
+  | Req_not_found of (int * request)
+  | Req_found of (request * Ocsigen_http_frame.result)
 
 and extension2 =
     (unit -> unit) ->
@@ -179,18 +329,19 @@ and extension2 =
       request_state ->
       (answer * Ocsigen_http_frame.cookieset) Lwt.t
 
+
 type extension = request_state -> answer Lwt.t
 
 
 type parse_fun = Simplexmlparser.xml list -> extension2
 
+
 type parse_host =
     Parse_host of
       (url_path ->
-        string * string * int * int ->
-          parse_host -> parse_fun -> Simplexmlparser.xml -> extension)
+         parse_host -> parse_fun -> Simplexmlparser.xml -> extension)
 
-let (hosts : (virtual_hosts * extension2) list ref) =
+let (hosts : (virtual_hosts * config_info * extension2) list ref) =
   ref []
 
 
@@ -201,14 +352,21 @@ let get_hosts () = !hosts
 
 
 (*****************************************************************************)
-let site_match site_path url =
+(* To give parameters to extensions: *)
+let dynlinkconfig = ref ([] : Simplexmlparser.xml list)
+let set_config s = dynlinkconfig := s
+let get_config () = !dynlinkconfig
+
+
+(*****************************************************************************)
+let site_match request site_path url =
   (* We are sure that there is no / at the end or beginning of site_path *)
   (* and no / at the beginning of url *)
   (* and no // or ../ inside both of them *)
   (* We return the subpath without / at beginning *)
   let rec aux site_path url =
     match site_path, url with
-      | [], [] -> raise Ocsigen_Is_a_directory
+      | [], [] -> raise (Ocsigen_Is_a_directory request)
       | [], p -> Some p
       | a::l, aa::ll when a = aa -> aux l ll
       | _ -> None
@@ -228,8 +386,11 @@ let add_to_res_cookies res cookies_to_set =
      Ocsigen_http_frame.res_cookies =
      Ocsigen_http_frame.add_cookies res.Ocsigen_http_frame.res_cookies cookies_to_set}
 
-let rec make_ext awake cookies_to_set req_state genfun f =
+let make_ext awake cookies_to_set req_state (genfun : extension) (genfun2 : extension2) =
+  genfun req_state
+  >>= fun res ->
   let rec aux cookies_to_set = function
+    | Ext_do_nothing -> genfun2 awake cookies_to_set req_state
     | Ext_found r ->
         awake ();
         r () >>= fun r' ->
@@ -237,33 +398,47 @@ let rec make_ext awake cookies_to_set req_state genfun f =
           | Req_found (ri, _) -> ri
           | Req_not_found (_, ri) -> ri
         in
-        f
-          awake
+        genfun2
+          Ocsigen_lib.id (* already awoken *)
           Ocsigen_http_frame.Cookies.empty
-          (Req_found (ri,
-                      fun () ->
-                        Lwt.return (add_to_res_cookies r' cookies_to_set)))
+          (Req_found (ri, add_to_res_cookies r' cookies_to_set))
+    | Ext_found_continue_with r ->
+        awake ();
+        r () >>= fun (r', req) ->
+        genfun2
+          Ocsigen_lib.id (* already awoken *)
+          Ocsigen_http_frame.Cookies.empty
+          (Req_found (req, add_to_res_cookies r' cookies_to_set))
+    | Ext_found_continue_with' (r', req) ->
+        awake ();
+        genfun2
+          Ocsigen_lib.id (* already awoken *)
+          Ocsigen_http_frame.Cookies.empty
+          (Req_found (req, add_to_res_cookies r' cookies_to_set))
     | Ext_next e ->
         let ri = match req_state with
           | Req_found (ri, _) -> ri
           | Req_not_found (_, ri) -> ri
         in
-        f awake cookies_to_set (Req_not_found (e, ri))
+        genfun2 awake cookies_to_set (Req_not_found (e, ri))
     | Ext_continue_with (ri, cook, e) ->
-        f
+        genfun2
           awake
           (Ocsigen_http_frame.add_cookies cook cookies_to_set)
           (Req_not_found (e, ri))
+    | Ext_found_stop _
     | Ext_stop_site _
     | Ext_stop_host _
     | Ext_stop_all _
-    | Ext_retry_with _ as res -> Lwt.return (res, cookies_to_set)
+    | Ext_retry_with _ as res ->
+        Lwt.return (res, cookies_to_set)
     | Ext_sub_result sr ->
         sr awake cookies_to_set req_state
         >>= fun (res, cookies_to_set) ->
         aux cookies_to_set res
   in
-  genfun req_state >>= aux cookies_to_set
+  aux cookies_to_set res
+
 
 (*****************************************************************************)
 let fun_beg = ref (fun () -> ())
@@ -273,7 +448,6 @@ let fun_exn = ref (fun exn -> (raise exn : string))
 let rec default_parse_config
     (host : virtual_hosts)
     prevpath
-    ((defcharset, a, b, c) as defcharsetetc)
     (Parse_host parse_host)
     (parse_fun : parse_fun) = function
   | Simplexmlparser.Element ("site", atts, l) ->
@@ -299,56 +473,68 @@ let rec default_parse_config
               (Ocsigen_config.Config_file_error ("Wrong attribute for <site>: "^s))
       in
       let charset, dir = parse_site_attrs (None, None) atts in
-      let charsetetc = match charset with
-      | None -> defcharsetetc
-      | Some charset -> (charset, a, b, c)
-      in
       let path =
         prevpath@
         Ocsigen_lib.remove_slash_at_end
           (Ocsigen_lib.remove_slash_at_beginning
              (Ocsigen_lib.remove_dotdot (Neturl.split_path dir)))
       in
-      let parse_site = make_parse_site path charsetetc parse_host l in
+      let parse_config = make_parse_config path parse_host l in
       let ext awake cookies_to_set =
         function
           | Req_found (ri, res) ->
-              Lwt.return (Ext_found res, cookies_to_set)
+              Lwt.return (Ext_found_continue_with' (res, ri), cookies_to_set)
           | Req_not_found (e, oldri) ->
-              match site_match path oldri.ri_full_path with
+              match site_match oldri path oldri.request_info.ri_full_path with
               | None ->
                   Ocsigen_messages.debug (fun () ->
                     "site \""^
-                    (Ocsigen_lib.string_of_url_path path)^
+                    (Ocsigen_lib.string_of_url_path ~encode:true path)^
                     "\" does not match url \""^
-                    (Ocsigen_lib.string_of_url_path oldri.ri_full_path)^
+                    (Ocsigen_lib.string_of_url_path ~encode:true
+                       oldri.request_info.ri_full_path)^
                     "\".");
                   Lwt.return (Ext_next e, cookies_to_set)
               | Some sub_path ->
                   Ocsigen_messages.debug (fun () ->
                     "-------- site found: url \""^
-                    (Ocsigen_lib.string_of_url_path oldri.ri_full_path)^
+                    (Ocsigen_lib.string_of_url_path ~encode:true
+                       oldri.request_info.ri_full_path)^
                     "\" matches \""^
-                    (Ocsigen_lib.string_of_url_path path)^"\".");
+                    (Ocsigen_lib.string_of_url_path ~encode:true path)^"\".");
                   let ri = {oldri with
-                            ri_sub_path = sub_path;
-                            ri_sub_path_string =
-                            Ocsigen_lib.string_of_url_path sub_path}
+                              request_info =
+                                { oldri.request_info with
+                                    ri_sub_path = sub_path;
+                                    ri_sub_path_string =
+                                    Ocsigen_lib.string_of_url_path
+                                      ~encode:true sub_path} }
                   in
-                  parse_site awake cookies_to_set (Req_not_found (e, ri))
+                  parse_config awake cookies_to_set (Req_not_found (e, ri))
                   >>= function
                       (* After a site, we turn back to old ri *)
-                    | (Ext_stop_site (cs, err), cookies_to_set) ->
-                        Lwt.return
-                          (Ext_continue_with (oldri, cs, err), cookies_to_set)
+                    | (Ext_stop_site (cs, err), cookies_to_set)
                     | (Ext_continue_with (_, cs, err), cookies_to_set) ->
                         Lwt.return
                           (Ext_continue_with (oldri, cs, err), cookies_to_set)
+                    | (Ext_found_continue_with r, cookies_to_set) ->
+                        awake ();
+                        r () >>= fun (r', req) ->
+                        Lwt.return
+                          (Ext_found_continue_with' (r', oldri), cookies_to_set)
+                    | (Ext_found_continue_with' (r, req), cookies_to_set) ->
+                        Lwt.return
+                          (Ext_found_continue_with' (r, oldri), cookies_to_set)
+                    | (Ext_do_nothing, cookies_to_set) ->
+                        Lwt.return 
+                          (Ext_continue_with (oldri, 
+                                              Ocsigen_http_frame.Cookies.empty,
+                                              e), cookies_to_set)
                     | r -> Lwt.return r
       in
       (function
-        | Req_found (_, r) ->
-            Lwt.return (Ext_found r)
+        | Req_found (ri, r) ->
+            Lwt.return (Ext_found_continue_with' (r, ri))
         | Req_not_found (err, ri) ->
             Lwt.return (Ext_sub_result ext))
   | Simplexmlparser.Element (tag,_,_) ->
@@ -356,54 +542,46 @@ let rec default_parse_config
   | _ -> raise (Ocsigen_config.Config_file_error
                   ("Unexpected content inside <host>"))
 
-and make_parse_site path charsetetc parse_host l =
-  let f = parse_host path charsetetc (Parse_host parse_host) in
+and make_parse_config path parse_host l : extension2 =
+  let f = parse_host path (Parse_host parse_host) in
   (* creates all site data, if any *)
-  let rec parse_site = function
+  let rec parse_config : _ -> extension2 = function
     | [] ->
         (fun (awake : unit -> unit) cookies_to_set -> function
           | Req_found (ri, res) ->
-              Lwt.return (Ext_found res, cookies_to_set)
+              Lwt.return (Ext_found_continue_with' (res, ri), cookies_to_set)
           | Req_not_found (e, ri) ->
-              Lwt.return (Ext_next e, cookies_to_set))
+              Lwt.return
+                (Ext_continue_with
+                   (ri, Ocsigen_http_frame.Cookies.empty, e), cookies_to_set))
+(* was Lwt.return (Ext_next e, cookies_to_set))
+   but to use make_parse_site with userconf,
+   we need to know current ri after parsing the sub-configuration.
+*)
     | xmltag::ll ->
         try
-          let genfun =
-            f
-              parse_site
-              xmltag
-          in
-          let genfun2 = parse_site ll in
+          let genfun = f parse_config xmltag in
+          let genfun2 = parse_config ll in
           fun awake cookies_to_set req_state ->
             make_ext awake cookies_to_set req_state genfun genfun2
         with
         | Bad_config_tag_for_extension t ->
-            ignore
-              (Ocsigen_messages.errlog
-                 ("Unexpected tag <"^t^"> inside <site dir=\""^
-                  (Ocsigen_lib.string_of_url_path path)^"\"> (ignored)"));
-            parse_site ll
-        | Ocsigen_config.Config_file_error t
-        | Error_in_config_file t ->
-            ignore
-              (Ocsigen_messages.errlog
-                 ("Error while parsing configuration file: "^
-                  t^" (ignored)"));
-            parse_site ll
+            (* This case happens only if no extension has recognized the
+               tag at all *)
+            badconfig
+              "Unexpected tag <%s> inside <site dir=\"%s\">" t
+              (Ocsigen_lib.string_of_url_path ~encode:true path)
+        | Error_in_config_file _ as e -> raise e
         | e ->
-            ignore
-              (Ocsigen_messages.errlog
-                 ("Error while parsing configuration file: "^
-                  (try
-                     !fun_exn e
-                   with e -> Ocsigen_lib.string_of_exn e)^
-                  " (ignored)"));
-            parse_site ll
+            badconfig
+              "Error while parsing configuration file: %s"
+              (try !fun_exn e
+               with e -> Ocsigen_lib.string_of_exn e)
   in
   !fun_beg ();
   let r =
     try
-      parse_site l
+      parse_config l
     with e -> !fun_end (); raise e
 (*VVV May be we should avoid calling fun_end after parinf user config files
   (with extension userconf) ... See eliommod.ml
@@ -414,67 +592,100 @@ and make_parse_site path charsetetc parse_host l =
 
 (*****************************************************************************)
 
+type userconf_info = {
+  localfiles_root : string;
+}
 
-let register_extension,
-  parse_site_item,
-  parse_user_site_item,
-  get_beg_init,
-  get_end_init,
-  get_init_exn_handler =
-  let fun_site = ref default_parse_config in
-  let user_fun_site = ref default_parse_config in
+type parse_config = virtual_hosts -> parse_config_aux
+and parse_config_user = userconf_info -> parse_config
+and parse_config_aux =
+    url_path -> parse_host ->
+      (parse_fun -> Simplexmlparser.xml ->
+         extension
+      )
+
+
+
+let user_extension_void_fun_site : parse_config_user = fun _ _ _ _ _ -> function
+  | Simplexmlparser.Element (t, _, _) -> raise (Bad_config_tag_for_extension t)
+  | _ -> raise (Error_in_config_file "Unexpected data in config file")
+
+let extension_void_fun_site : parse_config = fun _ _ _ _ -> function
+  | Simplexmlparser.Element (t, _, _) -> raise (Bad_config_tag_for_extension t)
+  | _ -> raise (Error_in_config_file "Unexpected data in config file")
+
+
+let register_extension, parse_config_item, parse_user_site_item, get_beg_init, get_end_init, get_init_exn_handler =
+  let ref_fun_site = ref default_parse_config in
+  let ref_user_fun_site = ref (fun (_ : userconf_info) -> default_parse_config) in
 
   ((* ********* register_extension ********* *)
-     (fun ?(respect_pipeline=false)
-         new_fun_site
-         new_user_fun_site
-         begin_init
-         end_init
-         handle_exn ->
+    (fun
+         ?fun_site
+         ?user_fun_site
+         ?begin_init
+         ?end_init
+         ?(exn_handler=raise)
+         ?(respect_pipeline=false)
+       ()
+       ->
 
        if respect_pipeline then Ocsigen_config.set_respect_pipeline ();
 
-       let old_fun_site = !fun_site in
-       fun_site :=
-         (fun host ->
-           let oldf = old_fun_site host in
-           let newf = new_fun_site host in
-           fun path charsetetc parse_host ->
-             let oldf = oldf path charsetetc parse_host in
-             let newf = newf path charsetetc parse_host in
-             fun parse_site config_tag ->
-               try
-                 oldf parse_site config_tag
-               with
-               | Bad_config_tag_for_extension c -> newf parse_site config_tag
-         );
+       (match fun_site with
+         | None -> ()
+         | Some fun_site ->
+             let old_fun_site = !ref_fun_site in
+             ref_fun_site :=
+               (fun host ->
+                  let oldf = old_fun_site host in
+                  let newf = fun_site host in
+                  fun path parse_host ->
+                    let oldf = oldf path parse_host in
+                    let newf = newf path parse_host in
+                    fun parse_config config_tag ->
+                      try
+                        oldf parse_config config_tag
+                      with
+                        | Bad_config_tag_for_extension c -> 
+                            newf parse_config config_tag
+               ));
 
-       let old_fun_site = !user_fun_site in
-       user_fun_site :=
-         (fun host ->
-           let oldf = old_fun_site host in
-           let newf = new_user_fun_site host in
-           fun path charsetetc parse_host ->
-             let oldf = oldf path charsetetc parse_host in
-             let newf = newf path charsetetc parse_host in
-             fun parse_site config_tag ->
-               try
-                 oldf parse_site config_tag
-               with
-               | Bad_config_tag_for_extension c -> newf parse_site config_tag
-         );
+       (match user_fun_site with
+         | None -> ()
+         | Some user_fun_site ->
+             let old_fun_site = !ref_user_fun_site in
+             ref_user_fun_site :=
+               (fun path host ->
+                  let oldf = old_fun_site path host in
+                  let newf = user_fun_site path host in
+                  fun path parse_host ->
+                    let oldf = oldf path parse_host in
+                    let newf = newf path parse_host in
+                    fun parse_config config_tag ->
+                      try
+                        oldf parse_config config_tag
+                      with
+                        | Bad_config_tag_for_extension c -> 
+                            newf parse_config config_tag
+               ));
 
-       fun_beg := comp begin_init !fun_beg;
-       fun_end := comp end_init !fun_end;
+
+       (match begin_init with
+         | Some begin_init -> fun_beg := comp begin_init !fun_beg
+         | None -> ());
+       (match end_init with
+         | Some end_init -> fun_end := comp end_init !fun_end;
+         | None -> ());
        let curexnfun = !fun_exn in
-       fun_exn := fun e -> try curexnfun e with e -> handle_exn e),
+       fun_exn := fun e -> try curexnfun e with e -> exn_handler e),
 
 
-   (* ********* parse_site_item ********* *)
-   (fun host -> !fun_site host),
+   (* ********* parse_config_item ********* *)
+   (fun host -> !ref_fun_site host),
 
    (* ********* parse_user_site_item ********* *)
-   (fun host -> !user_fun_site host),
+   (fun host -> !ref_user_fun_site host),
 
    (* ********* get_beg_init ********* *)
    (fun () -> !fun_beg),
@@ -486,11 +697,28 @@ let register_extension,
    (fun () -> !fun_exn)
   )
 
+let default_parse_extension ext_name = function
+  | [] -> ()
+  | _ -> raise (Error_in_config_file
+                  (Printf.sprintf "Unexpected content found in configuration of extension %s: %s does not accept options" ext_name ext_name))
 
-let void_extension _ _ _ _ _ = function
-  | Simplexmlparser.Element (t, _, _) -> raise (Bad_config_tag_for_extension t)
-  | _ -> raise (Error_in_config_file "Unexpected data in config file")
-
+let register_extension 
+    ~name
+    ?fun_site
+    ?user_fun_site
+    ?begin_init
+    ?end_init
+    ?init_fun
+    ?exn_handler
+    ?respect_pipeline
+    () =
+  Ocsigen_loader.set_module_init_function name
+    (fun () ->
+       (match init_fun with
+          | None -> default_parse_extension name (get_config ())
+          | Some f -> f (get_config ()));
+       register_extension ?fun_site ?user_fun_site ?begin_init ?end_init
+         ?exn_handler ?respect_pipeline ())
 
 (*****************************************************************************)
 let start_initialisation, during_initialisation,
@@ -576,14 +804,14 @@ let do_for_site_matching host port ri =
       end
   in
 
-  let rec do2 sites cookies_to_set ri =
+  let rec do2 nb_retries sites cookies_to_set ri =
     let string_of_host_option = function
       | None -> "<no host>:"^(string_of_int port)
       | Some h -> h^":"^(string_of_int port)
     in
     let rec aux_host ri prev_err cookies_to_set = function
       | [] -> fail (Ocsigen_http_error (cookies_to_set, prev_err))
-      | (h, host_function)::l when host_match host port h ->
+      | (h, conf_info, host_function)::l when host_match host port h ->
           Ocsigen_messages.debug (fun () ->
             "-------- host found! "^
             (string_of_host_option host)^
@@ -591,13 +819,24 @@ let do_for_site_matching host port ri =
           host_function
             awake
             cookies_to_set
-            (Req_not_found (prev_err, ri))
+            (Req_not_found (prev_err, { request_info = ri;
+                                        request_config = conf_info }))
           >>= fun (res_ext, cookies_to_set) ->
           (match res_ext with
-          | Ext_found r ->
+          | Ext_found r
+          | Ext_found_stop r ->
               awake ();
               r () >>= fun r' ->
+              Lwt.return (add_to_res_cookies r' cookies_to_set)
+          | Ext_do_nothing ->
+              aux_host ri prev_err cookies_to_set l
+          | Ext_found_continue_with r ->
+              awake ();
+              r () >>= fun (r', _) ->
               return (add_to_res_cookies r' cookies_to_set)
+          | Ext_found_continue_with' (r, _) ->
+              awake ();
+              return (add_to_res_cookies r cookies_to_set)
           | Ext_next e ->
               aux_host ri e cookies_to_set l
                 (* try next site *)
@@ -610,20 +849,21 @@ let do_for_site_matching host port ri =
           | Ext_continue_with (_, cook, e) ->
               aux_host ri e
                 (Ocsigen_http_frame.add_cookies cook cookies_to_set) l
-          | Ext_retry_with (ri2, cook) ->
-              (*VVV not enough to detect loops *)
-              if ri != ri2 then
+          | Ext_retry_with (request2, cook) ->
+              if nb_retries < Ocsigen_config.get_maxretries ()
+              then
                 do2
+                  (nb_retries + 1)
                   (get_hosts ())
                   (Ocsigen_http_frame.add_cookies cook cookies_to_set)
-                  ri2
+                  request2.request_info
                   (* retry all *)
               else
                 fail Ocsigen_Looping_request
           | Ext_sub_result sr ->
               assert false
           )
-      | (h, _)::l ->
+      | (h, _, _)::l ->
           Ocsigen_messages.debug (fun () ->
             "-------- host = "^
             (string_of_host_option host)^
@@ -633,7 +873,7 @@ let do_for_site_matching host port ri =
   in
   Lwt.finalize
     (fun () ->
-      do2 (get_hosts ()) Ocsigen_http_frame.Cookies.empty ri
+      do2 0 (get_hosts ()) Ocsigen_http_frame.Cookies.empty ri
     )
     (fun () ->
        awake ();
@@ -645,9 +885,9 @@ let do_for_site_matching host port ri =
 
 
 
-
+(* used to modify the url in ri (for example for retrying after rewrite) *)
 let ri_of_url url ri =
-  let (host, _, url, url2, path, params, get_params) = parse_url url in
+  let (_, host, _, url, url2, path, params, get_params) = parse_url url in
   let host = match host with
     | Some h -> host
     | None -> ri.ri_host
@@ -656,8 +896,9 @@ let ri_of_url url ri =
    ri_url_string = url;
    ri_url = url2;
    ri_host = host;
-   ri_full_path_string = string_of_url_path path;
+   ri_full_path_string = string_of_url_path ~encode:true path;
    ri_full_path = path;
+     (* ri_original_full_path is not changed *)
    ri_sub_path = path;
    ri_get_params_string = params;
    ri_get_params = get_params;
@@ -679,17 +920,39 @@ let get_number_of_connected,
 
 
 
-(*****************************************************************************)
-(* To give parameters to extensions: *)
-let dynlinkconfig = ref ([] : Simplexmlparser.xml list)
-let set_config s = dynlinkconfig := s
-let get_config () = !dynlinkconfig
 
 
+(*****************************************************************************)
+(* Default hostname is either the Host header or the hostname set in 
+   the configuration file. *)
+let get_hostname req =
+  if Ocsigen_config.get_usedefaulthostname ()
+  then req.request_config.default_hostname
+  else match req.request_info.ri_host with
+    | None -> req.request_config.default_hostname
+    | Some host -> host
+
+
+(*****************************************************************************)
+(* Default port is either
+   - the port the server is listening at
+   - or the port in the Host header
+   - or the default port set in the configuration file. *)
+let get_port req =
+  if Ocsigen_config.get_usedefaulthostname ()
+  then (if req.request_info.ri_ssl
+        then req.request_config.default_httpsport
+        else req.request_config.default_httpport)
+  else match req.request_info.ri_port_from_host_field with
+    | None -> req.request_info.ri_server_port
+    | Some p -> p
+
 
 (*****************************************************************************)
 (* user directories *)
 
+exception NoSuchUser
+
 type ud_string = Nodir of string | Withdir of string * string * string
 
 let user_dir_regexp = Netstring_pcre.regexp "(.*)\\$u\\(([^\\)]*)\\)(.*)"
@@ -709,10 +972,55 @@ let replace_user_dir regexp dest pathstring =
   | Nodir dest ->
       Netstring_pcre.global_replace regexp dest pathstring
   | Withdir (s1, u, s2) ->
-      let s1 = Netstring_pcre.global_replace regexp s1 pathstring in
-      let u = Netstring_pcre.global_replace regexp u pathstring in
-      let s2 = Netstring_pcre.global_replace regexp s2 pathstring in
-      let userdir = (Unix.getpwnam u).Unix.pw_dir in
-      s1^userdir^s2
+      try
+        let s1 = Netstring_pcre.global_replace regexp s1 pathstring in
+        let u = Netstring_pcre.global_replace regexp u pathstring in
+        let s2 = Netstring_pcre.global_replace regexp s2 pathstring in
+        let userdir = (Unix.getpwnam u).Unix.pw_dir in
+        Ocsigen_messages.debug (fun () -> "User " ^ u);
+        s1^userdir^s2
+      with Not_found ->
+        Ocsigen_messages.debug (fun () -> "No such user " ^ u);
+        raise NoSuchUser
 
 
+(*****************************************************************************)
+(* Finding redirections *)
+
+exception Not_concerned
+
+let find_redirection regexp full_url dest
+    https host port 
+    get_params_string
+    sub_path_string
+    full_path_string
+    =
+  if full_url
+  then
+    match host with
+      | None -> raise Not_concerned
+      | Some host ->
+          let path =
+            match get_params_string with
+              | None -> full_path_string
+              | Some g -> full_path_string ^ "?" ^ g
+          in
+          let path =
+            Ocsigen_lib.make_absolute_url https host port ("/"^path)
+          in
+          (match Netstring_pcre.string_match regexp path 0 with
+             | None -> raise Not_concerned
+             | Some _ -> (* Matching regexp found! *)
+                 Netstring_pcre.global_replace regexp dest path
+          )
+  else
+    let path =
+      match get_params_string with
+        | None -> sub_path_string
+        | Some g -> sub_path_string ^ "?" ^ g
+    in
+    match Netstring_pcre.string_match regexp path 0 with
+      | None -> raise Not_concerned
+      | Some _ -> (* Matching regexp found! *)
+          Netstring_pcre.global_replace regexp dest path
+
