@@ -30,12 +30,19 @@ open Lwt
 open Ocsigen_lib
 
 exception Ocsigen_http_error of (Ocsigen_http_frame.cookieset * int)
-exception Ocsigen_Is_a_directory
-exception Ocsigen_malformed_url
 exception Ocsigen_Internal_Error of string
 
-exception Bad_config_tag_for_extension of string (** Try next extension *)
-exception Error_in_config_file of string (** Stop with an error message *)
+
+(** Xml tag not recognized by an extension (usually not a real error) *)
+exception Bad_config_tag_for_extension of string
+
+(** Error in a <site> tag inside the main ocsigen.conf file *)
+exception Error_in_config_file of string
+
+(** Option incorrect in a userconf file *)
+exception Error_in_user_config_file of string
+
+
 
 val badconfig : ('a, unit, string, 'b) format4 -> 'a
 (** Convenient function for raising Error_in_config_file exceptions with
@@ -45,13 +52,73 @@ val badconfig : ('a, unit, string, 'b) format4 -> 'a
 (** The type of URL paths. [["plop";"plip"]] corresponds to [plop/plip]. *)
 type url_path = string list
 
-val string_of_url_path : url_path -> string
+val string_of_url_path : encode:bool -> url_path -> string
 
 
 (* virtual hosts: *)
 type virtual_host_part = Text of string * int | Wildcard
 type virtual_hosts = ((virtual_host_part list) * int option) list
 
+
+(*****************************************************************************)
+
+(** Configuration to hide/forbid local files *)
+type do_not_serve = {
+  do_not_serve_regexps: string list;
+  do_not_serve_files: string list;
+  do_not_serve_extensions: string list;
+}
+
+
+exception IncorrectRegexpes of do_not_serve
+
+(** Compile a do_not_serve structure into a regexp. Raises
+    [IncorrectRegexpes] if the compilation fails. The result is
+    memoized for subsequent calls with the same argument *)
+val do_not_serve_to_regexp: do_not_serve -> Netstring_pcre.regexp
+
+val join_do_not_serve : do_not_serve -> do_not_serve -> do_not_serve
+
+
+
+(** Configuration options, passed to (and modified by) extensions *)
+type config_info = {
+  default_hostname: string;
+  default_httpport: int;
+  default_httpsport: int;
+
+  mime_assoc: Ocsigen_charset_mime.mime_assoc;
+
+  charset_assoc : Ocsigen_charset_mime.charset_assoc;
+
+  (** Default name to use as index file when a directory is requested.
+      Use [None] if no index should be tried. The various indexes
+      are tried in the given order. If no index is specified,
+      or the index does not exists, the content of the directory
+      might be listed, according to [list_directry_content] *)
+  default_directory_index : string list;
+
+  (** Should the list of files in a directory be displayed
+      if there is no index in this directory ? *)
+  list_directory_content : bool;
+
+  (** Should symlinks be followed when accessign a local file? *)
+  follow_symlinks: follow_symlink;
+
+  do_not_serve_404: do_not_serve;
+  do_not_serve_403: do_not_serve;
+
+}
+and follow_symlink =
+  | DoNotFollowSymlinks (** Never follow a symlink *)
+  | FollowSymlinksIfOwnerMatch (** Follow a symlink if the symlink and its
+                          target have the same owner *)
+  | AlwaysFollowSymlinks (** Always follow symlinks *)
+
+
+(*****************************************************)
+
+
 (** The files sent in the request *)
 type file_info = {tmp_filename: string; (** Where the file is stored on the server*)
                   filesize: int64; (** Size, in bytes *)
@@ -77,6 +144,8 @@ val client_id : client -> int
 val client_connection : client -> Ocsigen_http_com.connection
 (** Returns the connection *)
 
+type ifrange = IR_No | IR_Ifunmodsince of float | IR_ifmatch of string
+
 (** The request *)
 type request_info =
     {ri_url_string: string; (** full URL *)
@@ -86,10 +155,13 @@ type request_info =
      ri_ssl: bool; (** true if HTTPS, false if HTTP *)
      ri_full_path_string: string; (** full path of the URL *)
      ri_full_path: string list;   (** full path of the URL *)
+     ri_original_full_path_string: string;   (** full path of the URL, as first sent by the client. Should not be changed by extensions, even rewritemod. It is used to create relative links. *)
+     ri_original_full_path: string list;   (** full path of the URL, as first sent by the client. See below. *)
      ri_sub_path: string list;   (** path of the URL (only part concerning the site) *)
      ri_sub_path_string: string;   (** path of the URL (only part concerning the site) *)
      ri_get_params_string: string option; (** string containing GET parameters *)
-     ri_host: string option; (** Host field of the request (if any) *)
+     ri_host: string option; (** Host field of the request (if any), without port *)
+     ri_port_from_host_field: int option; (** Port in the host field of the request (if any) *)
      ri_get_params: (string * string) list Lazy.t;  (** Association list of get parameters *)
      ri_initial_get_params: (string * string) list Lazy.t;  (** Association list of get parameters, as sent by the browser (must not be modified by extensions) *)
      ri_post_params: (string * string) list Lwt.t Lazy.t; (** Association list of post parameters *)
@@ -104,7 +176,7 @@ type request_info =
      ri_cookies: string Ocsigen_http_frame.Cookievalues.t Lazy.t;  (** Cookies sent by the browser *)
      ri_ifmodifiedsince: float option;   (** if-modified-since field *)
      ri_ifunmodifiedsince: float option;   (** if-unmodified-since field *)
-     ri_ifnonematch: string list;   (** if-none-match field ( * and weak entity tags not implemented) *)
+     ri_ifnonematch: string list option;   (** if-none-match field ( * and weak entity tags not implemented) *)
      ri_ifmatch: string list option;   (** if-match field ( * not implemented) *)
      ri_content_type: ((string * string) * (string * string) list) option; (** Content-Type HTTP header *)
      ri_content_type_string: string option; (** Content-Type HTTP header *)
@@ -123,14 +195,26 @@ type request_info =
                                       extensions
                                    *)
      ri_client: client; (** The request connection *)
+     ri_range: ((int64 * int64) list * int64 option * ifrange) option Lazy.t; 
+     (** Range HTTP header. [None] means all the document. 
+         List of intervals + possibly from an index to the end of the document.
+     *)
    }
-
 (** If you force [ri_files] or [ri_post_params], the request is fully read,
    so it is not possible any more to read it from [ri_http_frame]
    (and vice versa).
  *)
+and request = {
+  request_info: request_info;
+  request_config: config_info;
+}
+
+exception Ocsigen_Is_a_directory of request
+
 
 type answer =
+  | Ext_do_nothing
+      (** Do nothing *)
   | Ext_found of (unit -> Ocsigen_http_frame.result Lwt.t)
       (** "OK stop! I will take the page.
           You can start the following request of the same pipelined connection.
@@ -143,6 +227,8 @@ type answer =
           In that case, wait to be sure that the new request will not
           overtake this one.
       *)
+  | Ext_found_stop of (unit -> Ocsigen_http_frame.result Lwt.t)
+      (** Found but do not try next extensions *)
   | Ext_next of int (** Page not found. Try next extension.
                         The integer is the HTTP error code.
                         It is usally 404, but may be for ex 403 (forbidden)
@@ -168,7 +254,7 @@ type answer =
                         do not .
                         The integer is the HTTP error code, usally 403.
                      *)
-  | Ext_continue_with of (request_info * Ocsigen_http_frame.cookieset * int)
+  | Ext_continue_with of (request * Ocsigen_http_frame.cookieset * int)
         (** Used to modify the request before giving it to next extension.
             The extension returns the request_info (possibly modified)
             and a set of cookies if it wants to set or cookies
@@ -179,7 +265,7 @@ type answer =
             The integer is usually equal to the error code received
             from preceding extension (but you may want to modify it).
          *)
-  | Ext_retry_with of request_info * Ocsigen_http_frame.cookieset
+  | Ext_retry_with of request * Ocsigen_http_frame.cookieset
         (** Used to retry all the extensions with a new request_info.
             The extension returns the request_info (possibly modified)
             and a set of cookies if it wants to set or cookies
@@ -195,16 +281,24 @@ type answer =
             the parsing function (of type [parse_fun]),
             that will return something of type [extension2].
         *)
+  | Ext_found_continue_with of 
+      (unit -> (Ocsigen_http_frame.result * request) Lwt.t)
+        (** Same as [Ext_found] but may modify the request. *)
+  | Ext_found_continue_with' of (Ocsigen_http_frame.result * request)
+        (** Same as [Ext_found_continue_with] but does not allow to delay
+            the computation of the page. You should probably not use it,
+            but for output filters.
+        *)
 
 and request_state =
-  | Req_not_found of (int * request_info)
-  | Req_found of (request_info * (unit -> Ocsigen_http_frame.result Lwt.t))
+  | Req_not_found of (int * request)
+  | Req_found of (request * Ocsigen_http_frame.result)
 
 and extension2 =
-    (unit -> unit) ->
-      Ocsigen_http_frame.cookieset ->
-        request_state ->
-          (answer * Ocsigen_http_frame.cookieset) Lwt.t
+  (unit -> unit) ->
+  Ocsigen_http_frame.cookieset ->
+  request_state ->
+  (answer * Ocsigen_http_frame.cookieset) Lwt.t
 
 type extension = request_state -> answer Lwt.t
 (** For each <site> tag in the configuration file,
@@ -222,11 +316,23 @@ type extension = request_state -> answer Lwt.t
 
 type parse_fun = Simplexmlparser.xml list -> extension2
 
+(** Type of the functions parsing the content of a <host> tag *)
 type parse_host
 
-(**
-   For each extension generating pages, we register four functions:
-   - a function taking
+
+(** Information received by extensions accepting userconf files.
+
+   The parameter [localfiles_root] is an absolute path to the
+   directory that the user is allowed to serve. This is used
+   by staticmod, to disallow the user from allowing access to
+   outside of this directory
+*)
+type userconf_info = {
+  localfiles_root : string;
+}
+
+(** [parse_config] is the type of the functions parsing a <site> tag
+    (and returning an extension).  Those are functions taking
    {ul
      {- the name of the virtual <host>}}
      that will be called for each <host>,
@@ -241,71 +347,79 @@ type parse_host
    {ul
      {- raise [Bad_config_tag_for_extension] if it does not recognize that tag}
      {- return something of type [extension] (filter or page generator)}}
-   - a function of same type, that will be called every time user configuration
-    files are parsed (if userconf is enabled).
-    It must define only safe options, for example it is not
-    safe to allow such options to load a cmo specified by a user, or to
-    execute a program, as this program will be executed by ocsigen's user.
-    Note that function will be called for every request, whereas the first one
-    is called only when starting or reloading the server.
-    If you do not want to allow users to use your extension,
-    use the predefined function [void_extension] (defines no option).
-   - a function that will be called at the beginning
-   of the initialisation phase (each time the config file is reloaded)
-   (Note that the extensions are not reloaded)
-   - a function that will be called at the end of the initialisation phase
-   of the server
-   - a function that will create an error message from the exceptions
-   that may be raised during the initialisation phase, and raise again
-   all other exceptions
 
-   If the optional parameter [?respect_pipeline] is [true], the extension
-   will ask the server to respect the order of the pipeline. That means that
-   it will wait to be sure that the previous request from the same connection
-   has been taken by an extension before giving a request to an extension.
-   Use this to write proxies extensions, when you want to be able to pipeline
-   the requests you to another server. It is false by default.
- *)
+    [parse_config_user] is the type of functions parsing a site tag
+    inside an userconf file. They take one more parameter, of type userconf_info
+*)
+type parse_config =
+  virtual_hosts -> parse_config_aux
+and parse_config_user =
+    userconf_info -> parse_config
+and parse_config_aux =
+    url_path -> parse_host ->
+      (parse_fun -> Simplexmlparser.xml ->
+         extension
+      )
+
+
+(** For each extension generating pages, we register its name and six functions:
+- a function [fun_site] of type [parse_config]. This function
+will be responsible for handling the options of the configuration
+files that are recognized by the extension, and potentially generating
+a page.
+- a function [user_fun_site] of type [parse_user_config] which has the
+same role as [fun_site], but inside userconf files. Specify nothing
+if your extension is disallowed in userconf files. Otherwise, compared
+to [fun_site], you can selectively disallow some options,
+as [user_fun_site] must define only safe options (for example it is not
+safe to allow such options to load a cmo specified by a user, or to
+execute a program, as this program will be executed by ocsigen's user).
+Note that [user_fun_site] will be called for every request, whereas the
+[fun_site] is called only when starting or reloading the server.
+- a function [begin_init] that will be called at the beginning
+of the initialisation phase of each site, and each time the config file is
+reloaded.
+- a function [end_init] that will be called at the end of the initialisation
+phase of each site
+- a function [init_fun] that will be called just before registering the
+extension, taking as parameter the configuration options between
+[<extension>] and [</extension>]. This allows to give configuration options
+to extensions. If no function is supplied, the extension is supposed to
+accept no option (and loading will fail if an option is supplied)
+- a function [exn_handler] that will create an error message from the
+exceptions that may be raised during the initialisation phase, and raise again
+all other exceptions
+
+Moreover, if the optional parameter [?respect_pipeline] is [true],
+the extension will ask the server to respect the order of the
+pipeline. That means that it will wait to be sure that the previous
+request from the same connection has been taken by an extension
+before giving a request to an extension.  Use this to write proxies
+extensions, when you want to be able to pipeline the requests you
+to another server. It is false by default.
+
+*)
 val register_extension :
-  ?respect_pipeline: bool ->
-  (virtual_hosts ->
-     url_path ->
-       string * string * int * int ->
-         parse_host ->
-           parse_fun ->
-             Simplexmlparser.xml ->
-               extension) ->
-  (virtual_hosts ->
-     url_path ->
-       string * string * int * int ->
-         parse_host ->
-           parse_fun ->
-             Simplexmlparser.xml ->
-               extension) ->
-  (unit -> unit) ->
-  (unit -> unit) ->
-  (exn -> string) ->
-  unit
+  name:string ->
+  ?fun_site:parse_config ->
+  ?user_fun_site:parse_config_user ->
+  ?begin_init:(unit -> unit) ->
+  ?end_init:(unit -> unit) ->
+  ?init_fun:(Simplexmlparser.xml list -> unit) ->
+  ?exn_handler:(exn -> string) ->
+  ?respect_pipeline:bool ->
+  unit -> unit
 
 
-(** A predefined function to be passed to {!Ocsigen_extensions.register_extension}
-    that defines no option.
- *)
-val void_extension :
-    virtual_hosts ->
-      url_path ->
-        string * string * int * int ->
-          parse_host ->
-            parse_fun ->
-              Simplexmlparser.xml ->
-                extension
-
-
-(** While loading an extension,
-    get the configuration tree between <dynlink></dynlink>*)
-val get_config : unit -> Simplexmlparser.xml list
-
+(** Returns the hostname to be used for absolute links or redirections.
+    It is either the Host header or the hostname set in 
+    the configuration file. *)
+val get_hostname : request -> string
 
+(** Returns the port to be used for absolute links or redirections.
+    It is either the port the server is listening at or the default port set in
+    the configuration file. *)
+val get_port : request -> int
 
 (** Parsing URLs.
    This allows to modify the URL in the request_info.
@@ -314,7 +428,11 @@ val get_config : unit -> Simplexmlparser.xml list
 val ri_of_url : string -> request_info -> request_info
 
 
-(** User directories *)
+(** {3 User directories} *)
+
+(** Exception raised when an non-existing user is found *)
+
+exception NoSuchUser
 
 (** The type for string that may contain a $u(...) *)
 type ud_string
@@ -324,33 +442,28 @@ val parse_user_dir : string -> ud_string
 val replace_user_dir : Netstring_pcre.regexp -> ud_string -> string -> string
 (** raises [Not_found] is the directory does not exist *)
 
+
+
+(** {3 Regular expressions for redirections} *)
+exception Not_concerned
+
+val find_redirection :
+  Netstring_pcre.regexp ->
+  bool ->
+  string ->
+  bool ->
+  string option -> int -> string option -> string -> string -> string
+
 (**/**)
 
-val make_parse_site :
-  url_path ->
-  string * string * int * int ->
-  (url_path ->
-     string * string * int * int ->
-       parse_host -> parse_fun -> Simplexmlparser.xml -> extension) ->
-  parse_fun
+val make_parse_config : url_path -> parse_config_aux -> parse_fun
 
-val parse_site_item :
-  virtual_hosts ->
-  url_path ->
-  string * string * int * int ->
-  parse_host ->
-  parse_fun -> Simplexmlparser.xml -> extension
-  
-val parse_user_site_item :
-  virtual_hosts ->
-  url_path ->
-  string * string * int * int ->
-  parse_host ->
-  parse_fun -> Simplexmlparser.xml -> extension
+val parse_config_item : parse_config
+val parse_user_site_item : parse_config_user
 
-val set_hosts : (virtual_hosts * extension2) list -> unit
+val set_hosts : (virtual_hosts * config_info * extension2) list -> unit
 
-val get_hosts : unit -> (virtual_hosts * extension2) list
+val get_hosts : unit -> (virtual_hosts * config_info * extension2) list
 
 val do_for_site_matching :
     string option ->
