@@ -21,8 +21,6 @@
 
 (******************************************************************)
 (** Config file parsing *)
-(* 2006 07 : Add multiple servers -- Nataliya *)
-(* 2006 12 : Changes for extensions -- Vincent *)
 
 open Simplexmlparser
 open Ocsigen_config
@@ -35,7 +33,7 @@ let int_of_string tag s =
                                 " is not a valid integer."))
 
 (*****************************************************************************)
-let default_hostname =
+let default_default_hostname =
   let hostname = Unix.gethostname () in
   try
 (*VVV Is it ok? Is it reliable? *)
@@ -204,6 +202,70 @@ let preloadfile config () = Ocsigen_extensions.set_config config
 let postloadfile () = Ocsigen_extensions.set_config []
 
 
+(* Checking hostnames.  We make only make looze efforts.
+   See RFC 921 and 952 for further details *)
+let correct_hostname =
+  let regexp = Netstring_pcre.regexp "^[a-zA-Z0-9]+((\\.|-)[a-zA-Z0-9]+)*$" in
+  fun h -> Netstring_pcre.string_match regexp h 0 <> None
+
+(* Split the [host] field, first according to spaces, and then according
+   to wildcards '*' *)
+let parse_host_field host =
+  match host with
+    | None -> [[Ocsigen_extensions.Wildcard], None] (* default = "*:*" *)
+    | Some s ->
+        let parse_one_host ss =
+          let host, port =
+            try
+              let dppos = String.index ss ':'
+              and len = String.length ss in
+              let host = String.sub ss 0 dppos
+              and port =
+                match String.sub ss (dppos+1) ((len - dppos) - 1) with
+                  | "*" -> None
+                  | p -> Some (int_of_string "host" p)
+              in host, port
+            with
+              | Not_found -> ss, None
+              | Failure _ ->
+                  raise (Config_file_error "bad port number")
+          in
+          let split_host = function
+            | Netstring_str.Delim _ -> Ocsigen_extensions.Wildcard
+            | Netstring_str.Text t ->
+                Ocsigen_extensions.Text (t, String.length t)
+          in
+          List.map split_host
+            (Netstring_str.full_split (Netstring_str.regexp "[*]+") host),
+          port
+        in
+        List.map parse_one_host
+          (Netstring_str.split (Netstring_str.regexp "[ \t]+") s)
+
+
+(* Extract a default hostname from the "host" field if no default is provided *)
+let get_defaulthostname ~defaulthostname ~defaulthttpport ~host =
+    match defaulthostname with
+      | Some d -> d
+      | None ->
+          (* We look for a hostname without wildcard (second case) *)
+          (* Something more clever could be envisioned *)
+          let rec aux = function
+            | [] -> default_default_hostname
+            | ([Ocsigen_extensions.Text (t, _)], (Some 80 | None)) :: _ -> t
+            | _ :: q -> aux q
+          in
+          let host = aux host in
+          Ocsigen_messages.warning
+            ("While parsing config file, tag <host>: No defaulthostname, \
+              assuming it is \""^host^"\"");
+          if correct_hostname host then
+             host
+          else
+            raise (Ocsigen_config.Config_file_error
+                     ("Incorrect hostname " ^ host))
+
+
 (* Config file is parsed twice.
    This is the second parsing (site loading)
  *)
@@ -244,6 +306,9 @@ let parse_server isreloading c =
       | (Element ("mimefile" as st, [], p))::ll ->
           Ocsigen_config.set_mimefile (parse_string_tag st p);
           parse_server_aux ll
+      | (Element ("maxretries" as st, [], p))::ll ->
+          set_maxretries (int_of_string st (parse_string_tag st p));
+          parse_server_aux ll
       | (Element ("timeout" as st, [], p))::ll
 (*VVV timeout: backward compatibility with <= 0.99.4 *)
       | (Element ("clienttimeout" as st, [], p))::ll ->
@@ -280,6 +345,12 @@ let parse_server isreloading c =
       | (Element ("debugmode", [], []))::ll ->
           set_debugmode true;
           parse_server_aux ll
+      | (Element ("usedefaulthostname", [], []))::ll ->
+          set_usedefaulthostname true;
+          parse_server_aux ll
+      | (Element ("disablepartialrequests", [], []))::ll ->
+          set_disablepartialrequests true;
+          parse_server_aux ll
       | (Element ("respectpipeline", [], []))::ll ->
           set_respect_pipeline ();
           parse_server_aux ll
@@ -292,24 +363,36 @@ let parse_server isreloading c =
           let modules = match atts with
             | [] ->
                 raise
-                  (Config_file_error "missing module or findlib-package attribute in <extension>")
-            | [("module", s)] -> [s]
-            | [("findlib-package", s)] -> Ocsigen_loader.findfiles s
+                  (Config_file_error "missing module, name or findlib-package attribute in <extension>")
+            | [("name", s)] -> `Name s
+            | [("module", s)] -> `Files [s]
+            | [("findlib-package", s)] -> `Files (Ocsigen_loader.findfiles s)
             | _ ->
                 raise (Config_file_error "Wrong attribute for <extension>")
-          in
-          Ocsigen_loader.loadfiles (preloadfile l) postloadfile false modules;
+          in begin
+            match modules with
+              | `Files modules ->
+                  Ocsigen_loader.loadfiles (preloadfile l) postloadfile false modules;
+              | `Name name ->
+                  Ocsigen_loader.init_module (preloadfile l) postloadfile false name
+          end;
           parse_server_aux ll
       | (Element ("library", atts, l))::ll ->
           let modules = match atts with
             | [] ->
                 raise
                   (Config_file_error "missing module or findlib-package attribute in <library>")
-            | [("module", s)] -> [s]
-            | [("findlib-package", s)] -> Ocsigen_loader.findfiles s
+            | [("name", s)] -> `Name s
+            | [("module", s)] -> `Files [s]
+            | [("findlib-package", s)] -> `Files (Ocsigen_loader.findfiles s)
             | _ -> raise (Config_file_error "Wrong attribute for <library>")
-          in
-          Ocsigen_loader.loadfiles (preloadfile l) postloadfile true modules;
+          in begin
+            match modules with
+              | `Files modules ->
+                  Ocsigen_loader.loadfiles (preloadfile l) postloadfile true modules;
+              | `Name name ->
+                  Ocsigen_loader.init_module (preloadfile l) postloadfile true name
+          end;
           parse_server_aux ll
       | (Element ("host", atts, l))::ll ->
           let rec parse_attrs ((name, 
@@ -318,8 +401,7 @@ let parse_server isreloading c =
                                 defaulthttpport, 
                                 defaulthttpsport) as r) = function
             | [] -> r
-            | ("name", s)::suite (*VVV deprecated!! remove it in 1.2 *)
-            | ("aliases", s)::suite ->
+            | ("hostfilter", s)::suite ->
                 (match name with
                 | None -> parse_attrs ((Some s), charset, 
                                        defaulthostname, 
@@ -335,14 +417,17 @@ let parse_server isreloading c =
                                        defaulthttpsport) suite
                 | _ -> raise (Ocsigen_config.Config_file_error
                                 ("Duplicate attribute charset in <host>")))
-            | ("defaulthostname", s)::suite (*VVV deprecated!! remove it in 1.2 *)
-(*VVV may be (in 1.3 sqq):            | ("name", s)::suite *)
-            | ("hostname", s)::suite ->
+            | ("defaulthostname", s)::suite ->
                 (match defaulthostname with
-                | None -> parse_attrs (name, charset, 
-                                       (Some s), 
-                                       defaulthttpport, 
-                                       defaulthttpsport) suite
+                | None ->
+                    if correct_hostname s then
+                      parse_attrs (name, charset,
+                                   (Some s),
+                                   defaulthttpport,
+                                   defaulthttpsport) suite
+                    else
+                      raise (Ocsigen_config.Config_file_error
+                               ("Incorrect hostname " ^ s))
                 | _ -> raise (Ocsigen_config.Config_file_error
                                 ("Duplicate attribute defaulthostname in <host>")))
             | ("defaulthttpport", s)::suite ->
@@ -365,75 +450,15 @@ let parse_server isreloading c =
                 raise (Ocsigen_config.Config_file_error
                          ("Wrong attribute for <host>: "^s))
           in
-          let host, 
-            charset, 
-            defaulthostname, 
-            defaulthttpport, 
-            defaulthttpsport = 
-            parse_attrs (None, None, None, None, None) atts 
+          let host, charset, defaulthostname, defaulthttpport,defaulthttpsport =
+            parse_attrs (None, None, None, None, None) atts
           in
-          let host = match host with
-          | None -> [[Ocsigen_extensions.Wildcard], None] (* default = "*:*" *)
-          | Some s ->
-              List.map
-                (fun ss ->
-                  let host, port =
-                    try
-                      let dppos = String.index ss ':' in
-                      let len = String.length ss in
-                      ((String.sub ss 0 dppos),
-                       match String.sub ss (dppos+1) ((len - dppos) - 1) with
-                         | "*" -> None
-                         | p -> Some (int_of_string "host" p))
-                    with
-                      | Not_found -> ss, None
-                      | Failure _ ->
-                          raise (Config_file_error "bad port number")
-                  in
-                  ((List.map
-                      (function
-                         | Netstring_str.Delim _ -> Ocsigen_extensions.Wildcard
-                         | Netstring_str.Text t ->
-                            Ocsigen_extensions.Text (t, String.length t))
-                      (Netstring_str.full_split (Netstring_str.regexp "[*]+")
-                         host)),
-                   port))
-                (Netstring_str.split (Netstring_str.regexp "[ \t]+") s)
-          in
-          let charset = match charset with
-          | None -> Ocsigen_config.get_default_charset ()
-          | Some charset -> Some charset
-          in
-          let charset = match charset with
-          | None -> "utf-8"
-          | Some charset -> charset
-          in
-          let defaultdefaulthostname = default_hostname in
-          let defaulthostname = match defaulthostname with
-            | Some d -> d
-            | None ->
-                try
-                  (match
-                    fst
-                      (List.find 
-                         (* We look for a hostname without wildcard *)
-                         (*VVV one could do sthg more clever *)
-                         (function
-                            | ([Ocsigen_extensions.Text (t, _)], None) -> true
-                            | _ -> false
-                         ) 
-                         host)
-                   with
-                     | [Ocsigen_extensions.Text (t, _)] -> 
-                         Ocsigen_messages.warning
-                           ("While parsing config file, tag <host>: Assuming defaulthostname is \""^t^"\"");
-                         t
-                     | _ -> 
-                         defaultdefaulthostname)
-                with Not_found -> 
-                  Ocsigen_messages.warning
-                    ("While parsing config file, tag <host>: Assuming defaulthostname is \""^defaultdefaulthostname^"\"");
-                  defaultdefaulthostname
+          let host = parse_host_field host in
+          let charset =
+            match charset, Ocsigen_config.get_default_charset () with
+              | Some charset, _
+              | None, Some charset -> charset
+              | None, None -> "utf-8"
           in
           let defaulthttpport = match defaulthttpport with
             | None ->
@@ -441,20 +466,39 @@ let parse_server isreloading c =
                 with Failure _ -> 80)
             | Some p -> int_of_string "host" p
           in
+          let defaulthostname = get_defaulthostname
+            ~defaulthostname ~defaulthttpport ~host in
           let defaulthttpsport = match defaulthttpsport with
             | None ->
                 (try snd (List.hd (Ocsigen_config.get_sslports ())) 
                 with Failure _ -> 443)
             | Some p -> int_of_string "host" p
           in
-          let parse_host = Ocsigen_extensions.parse_site_item host in
-          let parse_site = 
-            Ocsigen_extensions.make_parse_site 
-              [] (charset, defaulthostname, defaulthttpport, defaulthttpsport)
-              parse_host 
+          let parse_host = Ocsigen_extensions.parse_config_item host in
+          let serve_everything = {
+            Ocsigen_extensions.do_not_serve_regexps = [];
+            do_not_serve_files = [];
+            do_not_serve_extensions = [];
+          } in
+          let conf = {
+            Ocsigen_extensions.default_hostname = defaulthostname;
+            default_httpport = defaulthttpport;
+            default_httpsport = defaulthttpsport;
+            mime_assoc = Ocsigen_charset_mime.default_mime_assoc ();
+            charset_assoc = Ocsigen_charset_mime.empty_charset_assoc
+              ~default:charset ();
+            default_directory_index = ["index.html"];
+            list_directory_content = false;
+            follow_symlinks = Ocsigen_extensions.FollowSymlinksIfOwnerMatch;
+            do_not_serve_404 = serve_everything;
+            do_not_serve_403 = serve_everything;
+          }
+          in
+          let parse_config =
+            Ocsigen_extensions.make_parse_config [] parse_host
           in
           (* default site for host *)
-          (host, parse_site l)::(parse_server_aux ll)
+          (host, conf, parse_config l)::(parse_server_aux ll)
       | (Element ("extconf", [("dir", dir)], []))::ll ->
           let one =
             try
@@ -499,6 +543,7 @@ let parse_server isreloading c =
   in Ocsigen_extensions.set_hosts (parse_server_aux c)
 
 
+
 (* Parsing <port> tags *)
 let parse_port =
   let all_ipv6 = Netstring_pcre.regexp "^\\[::\\]:([0-9]+)$" in
