@@ -994,7 +994,7 @@ let start_server () = try
     Ocsigen_messages.open_files ();
 
     Lwt_unix.run
-      (let wait_end_init = wait () in
+      (let wait_end_init, wait_end_init_awakener = wait () in
       (* Listening on all ports: *)
       List.iter
         (fun i ->
@@ -1125,11 +1125,11 @@ let start_server () = try
              end; f ())
       in ignore (f ());
 
-      wakeup wait_end_init ();
+      wakeup wait_end_init_awakener ();
 
       warning "Ocsigen has been launched (initialisations ok)";
 
-      wait ()
+      fst (wait ())
       )
   in
 
