@@ -1033,9 +1033,11 @@ module type T =
 (** [~encoding] is the official name of the external character set encoding that
     is used by [outs : string -> unit]. *)
 
-    val doctype :
-        [< `HTML_v03_02 | `HTML_v04_01 | `XHTML_01_00 | `XHTML_01_01 ]
-      -> string
+    type doctypes = 
+        [ `HTML_v03_02 | `HTML_v04_01 | `XHTML_01_00 | `XHTML_01_01
+        | `Doctype of string ]
+
+    val doctype : [< doctypes ] -> string
 
     val output : ?encode:(string -> string) -> ?encoding:string ->
       (string -> unit) -> html -> unit
@@ -1066,8 +1068,6 @@ module type T =
 (** Set the rowspan attribute for the element. *)
 
     val rewrite_hrefs : (string -> string) -> 'a elt -> 'a elt
-    val all_hrefs : 'a elt -> uri list
-    val all_anchors : 'a elt -> id list
 
 (*
     val amap : (string -> 'a attribs -> 'a attribs) -> 'b elt -> 'b elt
@@ -1805,6 +1805,10 @@ module Version =
       "<!DOCTYPE " ^ dt ^ " PUBLIC " ^
       String.concat " " (List.map (fun a -> "\"" ^ a ^ "\"") args) ^ ">\n"
 
+    type doctypes = 
+        [ `HTML_v03_02 | `HTML_v04_01 | `XHTML_01_00 | `XHTML_01_01
+        | `Doctype of string ]
+
     let doctype = function
       | `HTML_v03_02 ->
           compose_doctype "html" ["-//W3C//DTD HTML 3.2 Final//EN"]
@@ -1817,6 +1821,7 @@ module Version =
       | `XHTML_01_01 ->
           compose_doctype "html" ["-//W3C//DTD XHTML 1.1//EN";
                                   "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd"]
+      | `Doctype s -> s
 
     let no_break =
       ["title";
@@ -1872,9 +1877,6 @@ module Version =
     let rewrite_hrefs f =
       XML.amap (fun _ a -> XML.map_string_attrib ((=) "href") f a)
 
-    let all_hrefs = XML.all_string_attribs "href"
-    let all_anchors = XML.all_string_attribs ~is_elt:((=) "a") "id"
-
     let amap = XML.amap
     let amap1 = XML.amap1
 
@@ -1901,8 +1903,6 @@ module M_01_00 : T_01_00 =
     let output = M.output xhtml_version
     let pretty_print = M.pretty_print xhtml_version
     let validator_icon () = M.validator_icon xhtml_version
-    let all_anchors elt =
-      M.all_anchors elt @ XML.all_string_attribs ~is_elt:((=) "a") "name" elt
   end
 
 module M_01_01 : T_01_01 =
